CREATE TABLE absences (
  id mediumint(8) unsigned NOT NULL auto_increment,
  notes text NOT NULL,
  start_date timestamp(14) NOT NULL,
  end_date timestamp(14) NOT NULL,
  person mediumint(8) unsigned NOT NULL default '0',
  reason smallint NOT NULL, 
  count_on_timesheet bit NOT NULL default '1',
  PRIMARY KEY  (id),
  KEY person (person)
);

CREATE TABLE absence_backups (
  absence mediumint(8) unsigned NOT NULL default '0',
  notes text NOT NULL,
  person mediumint(8) unsigned NOT NULL default '0'
);

alter table companies add column payroll_contact mediumint(4) NOT NULL default '0';

CREATE TABLE company2project (
    company mediumint(4) unsigned NOT NULL,
    project mediumint(8) NOT NULL,
    role ENUM('Developer', 'Customer') NOT NULL default 'Customer'
);

alter table documents add KEY path (path);

CREATE TABLE events (
  id mediumint(8) unsigned NOT NULL auto_increment,
  title varchar(255) NOT NULL default '',
  description text NOT NULL,
  start_date timestamp(14) NOT NULL,
  end_date timestamp(14) NOT NULL,
  project mediumint(8) unsigned NOT NULL default '0',
  src mediumint(8) unsigned NOT NULL default '0',
  PRIMARY KEY  (id),
  KEY project (project)
);

CREATE TABLE event_comments (
  id mediumint(8) unsigned NOT NULL auto_increment,
  event mediumint(8) unsigned NOT NULL default '0',
  src mediumint(8) unsigned NOT NULL default '0',
  comment mediumtext NOT NULL,
  date timestamp(14) NOT NULL,
  PRIMARY KEY  (id),
  KEY event (event),
  KEY date (date)
);

CREATE TABLE know_votes (
  know mediumint(8) unsigned NOT NULL,
  person mediumint(8) unsigned NOT NULL default '0',
  vote smallint NOT NULL,
  PRIMARY KEY (know, person)
);

alter table meeting add column topic char(255) not null;

CREATE TABLE misc_time (
  id mediumint(8) unsigned NOT NULL auto_increment,
  project mediumint(8) unsigned NOT NULL,
  reason smallint NOT NULL, 
  person mediumint(8) unsigned NOT NULL,
  day date NOT NULL,
  worked float unsigned NOT NULL,
  comment mediumtext NOT NULL,
  PRIMARY KEY  (id)
);

alter table people_proj add column role ENUM('Developer', 'Customer') NOT NULL default 'Customer';
alter table people_proj drop primary key;
alter table people_proj add primary key (person, project, role);

alter table company2project change column role role ENUM('Developer', 'Customer') NOT NULL default 'Developer';
insert into company2project (company, project) select developer, id from projects;
alter table company2project change column role role ENUM('Developer', 'Customer') NOT NULL default 'Customer';
insert into company2project (company, project) select customer, id from projects;
alter table projects drop column customer;
alter table projects drop column developer;
alter table projects drop column iset;
alter table projects add column cost_code varchar(255) not null default '';

CREATE TABLE project_managers (
  id mediumint(8) unsigned NOT NULL auto_increment,
  project mediumint(8) unsigned NOT NULL default '0',
  manager varchar(255) NOT NULL default '',
  PRIMARY KEY  (id),
  KEY manager (manager),
  KEY project (project)
);

insert into project_managers (project, manager) select id, manager from projects;

alter table projects drop column manager;

CREATE TABLE project_settings (
  id mediumint(8) unsigned NOT NULL auto_increment,
  project mediumint(8) unsigned NOT NULL default '0',
  name varchar(255) NOT NULL default '',
  value varchar(255) NOT NULL default '',
  default_val varchar(255) NOT NULL default '',
  PRIMARY KEY  (id),
  KEY name (name),
  KEY project (project)
);

CREATE TABLE project_categories (
  id mediumint(9) NOT NULL auto_increment,
  cat_name varchar(255) NOT NULL default 'Unknown',
  person mediumint(8) unsigned NOT NULL default '0',
  project mediumint(9) NOT NULL default '0',
  PRIMARY KEY  (id)
);

insert into project_categories (id, cat_name, project) select catID, catName, ProjectID from task_category;
drop table task_category;

CREATE TABLE project_log (
  id mediumint(8) unsigned NOT NULL auto_increment,
  project mediumint(8) unsigned NOT NULL default '0',
  entry varchar(255) NOT NULL default '',
  date timestamp(14) NOT NULL,
  PRIMARY KEY  (id),
  KEY date (date)
);

CREATE TABLE project_milestones (
  id mediumint(9) NOT NULL auto_increment,
  name varchar(255) NOT NULL default 'Unknown',
  project mediumint(9) NOT NULL default '0',
  due_date date NOT NULL,
  PRIMARY KEY  (id)
);

CREATE TABLE request2task (
  request mediumint(8) unsigned NOT NULL default '0',
  task mediumint(8) unsigned NOT NULL default '0',
  PRIMARY KEY  (request,task)
);

insert into request2task (request, task) select id, atask from requests;
alter table requests drop column atask;

alter table requests drop column notify;
alter table requests add column notify_src bit NOT NULL default 1 after status;
alter table requests change column status status smallint NOT NULL default 1;
update requests set date=date, status = 666 where status = 2;
update requests set date=date, status = 2 where status = 3;
update requests set date=date, status = 3 where status = 666;
alter table requests add column priority char(255) NOT NULL default '';
alter table requests add column CatID mediumint(9) NOT NULL default '0';
alter table requests add column milestone mediumint(9) NOT NULL default '0';
alter table requests add column resolved_date timestamp(14) NOT NULL;
alter table requests add column initial_estimate TIME NOT NULL default 0;

CREATE TABLE request_submitters (
  id mediumint(8) unsigned NOT NULL auto_increment,
  request mediumint(8) unsigned NOT NULL default '0',
  person mediumint(8) unsigned NOT NULL default '0',
  PRIMARY KEY  (id),
  KEY request (person),
  KEY person (person)
);

insert into request_submitters (request, person) select id, src from requests;
alter table requests drop column src;

CREATE TABLE searches (
  id mediumint(8) unsigned NOT NULL auto_increment,
  name varchar(255) NOT NULL default '',
  person mediumint(8) unsigned NOT NULL default '0',
  project mediumint(8) unsigned NOT NULL default '0',
  type enum ('requests', 'tasks', 'documents', 'meetings', 'emails') NOT NULL,
  PRIMARY KEY  (id),
  KEY (name),
  KEY (person, project, type)
);

CREATE TABLE search_fields (
  search mediumint(8) unsigned NOT NULL,
  name varchar(255) NOT NULL default '',
  value text NOT NULL default '',
  KEY (search),
  KEY (name)
);

CREATE TABLE task_estimate_history (
  id bigint(20) unsigned NOT NULL auto_increment,
  taskID bigint(20) unsigned NOT NULL default '0',
  estimated time default 0 not null,
  date timestamp(14) NOT NULL,
  estimator mediumint(8) unsigned NOT NULL default '0',
  PRIMARY KEY  (id),
  KEY taskID (taskID)
);  

alter table task change column TaskDesc TaskDesc TEXT default '';

alter table task_progress_history add column remaining float NOT NULL default 0;
alter table task_progress_history add column person mediumint(8) unsigned NOT NULL default '0';

CREATE TABLE user_settings (
  id mediumint(8) unsigned NOT NULL auto_increment,
  person mediumint(8) unsigned NOT NULL default '0',
  project mediumint(8) unsigned NOT NULL default '0',
  type char(25) not null default '',
  name char(25) not null default '',
  value char(25) not null default '',
  PRIMARY KEY (id),
  KEY setting_key (person, project, type, name)  
);

INSERT INTO settings (name, value, default_val) VALUES ('allow_user_create_by_all','0','0');
INSERT INTO settings (name, value, default_val) VALUES ('show_request_ids','1','1');
INSERT INTO settings (name, value, default_val) VALUES ('timesheet_nonproj_reasons','Sick/Medical|Vacation|Stat Holiday|Training','Sick/Medical|Vacation|Stat Holiday|Training');
INSERT INTO settings (name, value, default_val) VALUES ('list_all_projects_even_if_no_access', '0','0');


INSERT INTO project_settings VALUES (1,0,'default_priority_level','4','4');
INSERT INTO project_settings VALUES (2,0,'display_milestones_for_requests','0','0');
INSERT INTO project_settings VALUES (3,0,'notify_whole_team_on_new_requests','0','0');
INSERT INTO project_settings VALUES (4,0,'only_pm_can_edit_requests','0','0');
INSERT INTO project_settings VALUES (5,0,'only_pm_can_see_request_estimate','0','0');
INSERT INTO project_settings VALUES (6,0,'only_pm_can_set_request_priority','0','0');
INSERT INTO project_settings VALUES (7,0,'pm_can_change_request_status_even_if_not_owner', '0', '0');
INSERT INTO project_settings VALUES (8,0,'prompt_set_request_status_on_task_complete', '2', '2');
INSERT INTO project_settings VALUES (9,0,'request_priority','Critical|High|Medium|Low|New','Critical|High|Medium|Low|New');
INSERT INTO project_settings VALUES (10,0,'request_status_options','Stalled','Stalled');
INSERT INTO project_settings VALUES (11,0,'software_project','0','0');
INSERT INTO project_settings VALUES (12,0,'task_status_hours_remaining','1','1');
INSERT INTO project_settings VALUES (13,0,'allow_multiple_requests_per_task','0','0');


