<?
#
# A minimal XML parsing library
#
# Bogdan Stancescu <bogdan@lanifex.com>, January 2002
# Revision 1.0
#
# A very VERY rudimentary XML parser - but we don't need more
# for the time being, so it should be ok.
#
# It was implemented this way in order to allow a wider range
# of servers to use OPT, while still storing the update data
# in an elegant manner.
#
/*
  For example, let's take this XML:

  <shelf>
    <book title="Hunting">
      <action>Read</action>
    </book>
    <book title="Fishing">
      <action>Burn</action>
    </book>
    <book title="Golfing" />
  </shelf>

  Parsing this would result in an array of the form:
  array(
    "shelf"=>array(
      0=>array(
        "book"=>array(
          "tags"=>array(
            "title"=>array(
              0=>"Hunting",
              1=>"Fishing",
              2=>"Golfing"
            )
          ),
          0=>array(
            "action"=>array(
              0=>"Read"
            )
          ),
          1=>array(
            "action"=>array(
              1=>"Burn"
            )
          )
        )
      )
    )
  )

  Looks awful, right? It's not that hard to use, though.
  If for example the value above is assigned to variable $bookshelves
  then if makes sense to write

  $myaction=$bookshelves["shelf"][$a]["book"][$b]["action"][$c];

  It makes sense because there may be more than one shelf, more than one book
  per shelf and more than one possible action for a certain book - since we want
  to know what the action $c for the book $b on the shelf $a is, we have to specify
  the whole line above...
*/
  function parse_tag($xml)
  {
    $daxml=$xml;
    // echo("Init XML: ".htmlspecialchars($xml)."<br>\n");
    while (((substr($daxml,0,2)=="<!") || (substr($daxml,0,2)=="<?")) &&
          strpos(substr($daxml,1),"<"))
    {
      $daxml=substr($daxml,strpos(substr($daxml,1),"<")+1);
      // echo("Parse: ".htmlspecialchars($daxml)."<br>\n");
    }
    $daxml=trim($daxml);
    while (strlen($daxml) && ($daxml[0]=="<"))
    {
      // echo("Parse2: ".htmlspecialchars($daxml)."<br>\n");
      $dapos=strpos($daxml,">")+1;
      $datag=substr($daxml,0,strpos($daxml,">")+1);
      if ($datag[1]=="/")
      {
        // finished parsing this tag branch
        return(array($array,substr($daxml,$dapos)));
      }
      $tagname=substr($datag,1,-1);
      $empty=false;
      if (strpos($tagname," "))
      {
        // Either attributes or an empty tag
        $tagrest=substr($tagname,strpos($tagname," ")+1);
        $tagname=substr($tagname,0,strpos($tagname," "));
        $attlist=explode(" ",$tagrest);
        while ($att=each($attlist))
        {
          if ($att[1]=="/")
          {
            // Empty tag
            $empty=true;
            $daxml=substr($daxml,$dapos);
          }
          else
          {
            // Don't put equal signs in tag values!
            list($attname,$attval)=explode("=",$att[1]);
            // Getting rid of the first and last chars in the value
            $array[$tagname]["tags"][$attname][]=substr($attval,1,-1);
          }
        }
      }
      if (!$empty)
      {
        list($array[$tagname][],$daxml)=parse_tag(substr($daxml,$dapos));
      }
      // echo("Received: ".htmlspecialchars($daxml)."<br>\n");
      $daxml=trim($daxml);
    }
    if ($array)
    {
      return($array);
    }
    else
    {
      return(array(substr($daxml,0,strpos($daxml,"<")),substr($daxml,strpos($daxml,">")+1)));
    }
  }

  function parse_xml($xml)
  {
    // echo("Got XML: ".htmlspecialchars($xml)."<br>\n");
    $array=array();
    $array=parse_tag($xml);
    return($array);
  }
?>
