<?
  #
  # Translations library
  #
  # Functions used for managing translation
  #
  # Bogdan Stancescu <bogdan@lanifex.com>, June 2002
  # Revision 1.0

  // Copies the translation files from the specified directory to the similar dir in OPT
  // Also parses the incoming files with msgfmt after copying
  // Used by import.php
  function copy_trans($dir,$reldir="") {
    global $CRM_path;
    $d=dir($dir);
    while($entry=$d->read()) {
      if ((substr($entry,0,1)==".") || (($reldir=="") && ($entry=="lang_info.xml"))) {
        // Already parsed lang_info.xml, so we can now unlink it.
        if ($entry=="lang_info.xml") {
          unlink("$dir/lang_info.xml");
        }
        continue;
      }
      if (is_dir("$dir/$entry")) {
        if (!is_dir("$CRM_path$reldir/$entry")) {
          if(!@mkdir("$CRM_path$reldir/$entry",0744)) {
            OPT_error(sprintf(__("Directory %s doesn't exist and it couldn't be created."),"$CRM_path$reldir/$entry"));
            return(false);
          }
        }
        if (!copy_trans("$dir/$entry","$reldir/$entry")) {
          return(false);
        } else {
          rmdir("$dir/$entry");
        }
      } else {
        if (!@copy("$dir/$entry","$CRM_path$reldir/$entry")) {
          OPT_error(sprintf(__("Unable to copy %s to %s!"),"$dir/$entry","$CRM_path$reldir/$entry"));
          return(false);
        } else {
          unlink("$dir/$entry");

          // Finally parsing the po file
          if (substr($entry,-3)==".po") {
            $parsedEntry=substr($entry,0,-3).".mo";
            @exec("msgfmt -o \"$CRM_path$reldir/$parsedEntry\" \"$CRM_path$reldir/$entry\" 2>&1",$dump,$err);
            if ($err) {
              OPT_error("msgfmt failed:<br>".implode("<br>",$dump));
              return(false);
            }
          }
        }
      }
    }
    return(true);
  }

  // Writes the ending sequence for the chmod files
  function finalize_dump() {
    global $temp_CRM_dir;
    if (!is_file("$temp_CRM_dir/restore_perm.sh")) {
      return(false);
    }
    $f=@fopen("$temp_CRM_dir/restore_perm.sh","a");
    if (!$f) {
      return(false);
    }
    fputs($f,"\n# Ending sequence\n\n");
    fputs($f,"mv $temp_CRM_dir/allow_perm.sh $temp_CRM_dir/~allow_perm.sh\n");
    fputs($f,"mv $temp_CRM_dir/restore_perm.sh $temp_CRM_dir/~restore_perm.sh\n");
    fclose($f);
    return(true);
  }

  // Creates the temp sh files for chmod-ing
  function dump_chmod($file,$really=true) {
    if (!$really) {
      return(false);
    }
    global $temp_CRM_dir;
    if (!is_file("$temp_CRM_dir/allow_perm.sh")) {
      $f=@fopen("$temp_CRM_dir/allow_perm.sh","w");
      if (!$f) {
        OPT_error(sprintf(__("Unable to write to %s!"),"$temp_CRM_dir/allow_perm.sh"));
        return(false);
      }
      fputs($f,"#!/bin/sh\n");
      fputs($f,"# Generated by OPT\n");
      fputs($f,"#\n");
      fputs($f,"# Sets the permissions for translation files in order to\n");
      fputs($f,"# allow OPT to read them for export. You can restore the\n");
      fputs($f,"# original permissions using restore_perms.sh in this directory.\n\n");
      fclose($f);
      chmod("$temp_CRM_dir/allow_perm.sh",0777);
    }
    if (!is_file("$temp_CRM_dir/restore_perm.sh")) {
      $f=@fopen("$temp_CRM_dir/restore_perm.sh","w");
      if (!$f) {
        OPT_error(sprintf(__("Unable to write to %s!"),"$temp_CRM_dir/restore_perm.sh"));
        return(false);
      }
      fputs($f,"#!/bin/sh\n");
      fputs($f,"# Generated by OPT\n");
      fputs($f,"#\n");
      fputs($f,"# Restores the permissions for the translation files\n");
      fputs($f,"# altered by allow_perms.sh in this directory.\n\n");
      fclose($f);
      chmod("$temp_CRM_dir/restore_perm.sh",0777);
    }
    $f1=@fopen("$temp_CRM_dir/allow_perm.sh","a");
    if (!$f1) {
      OPT_error(sprintf(__("Unable to write to %s!"),$temp_CRM_dir/allow_perm.sh));
      return(false);
    }
    $f2=@fopen("$temp_CRM_dir/restore_perm.sh","a");
    if (!$f1) {
      OPT_error(sprintf(__("Unable to write to %s!"),$temp_CRM_dir/restore_perm.sh));
      return(false);
    }
    fputs($f2,"chmod ".decoct(fileperms($file) & 0777)." \"$file\"\n");
    fputs($f1,"chmod ugo+r \"$file\"\n");
    fclose($f1);
    fclose($f2);
    return(true);
  }

  // Builds the encoding array to be written in lang_info.xml
  function add_encoding($lang) {
    global $OPT_lang_encodings;
    if (isset($OPT_lang_encodings[$lang])) {
      return(true);
    }
    $enc=getvalue("lang","encoding",$lang);
    // Even if the current system somehow doesn't have any encoding
    // set for this language, at least we set the array entry to ""
    // which makes subsequent calls to this functions skip the
    // database query above.
    $OPT_lang_encodings[$lang]=$enc;
  }

  // Builds the list of files for the selected dir
  // (this is the recursive part of the next function)
  function get_translation_list($what, $lang, $path, $list, $rel_path) {
    $d=dir($path);
    $generate_sh=true;
    while ($entry=$d->read()) {
      if ((!is_dir("$path$entry")) || ($entry[0]==".")) {
        continue;
      }
      if (!is_readable("$path$entry")) {
        OPT_error(sprintf(__("Unable to read %s!"),$path.$entry));
        $list["errors"]=1;
        continue;
      }
      if ($entry=="OPT_lang") {
        if ($what=="single") {
          if (is_file($cfile="$path$entry/$lang/LC_MESSAGES/OPT_local.po")) {
            $crel="$rel_path$entry/$lang/LC_MESSAGES/OPT_local.po";
            if(!is_readable($cfile)) {
              $list["errors"]=1;
              OPT_error(sprintf(__("Unable to read %s!"),"opt/$crel"));
              $generate_sh=dump_chmod($cfile,$generate_sh);
            } else {
              add_encoding($lang);
              $list["files"][]=$crel;
            }
          }
        } else {
          $d2=dir("$path$entry");
          while ($entry2=$d2->read()) {
            if(is_file($cfile="$path$entry/$entry2/LC_MESSAGES/OPT_local.po")) {
              $crel="$rel_path$entry/$entry2/LC_MESSAGES/OPT_local.po";
              if(!is_readable($cfile)) {
                $list["errors"]=1;
                OPT_error(sprintf(__("Unable to read %s!"),"opt/$crel"));
                $generate_sh=dump_chmod($cfile,$generate_sh);
              } else {
                add_encoding($entry2);
                $list["files"][]=$crel;
              }
            }
          }
        }
      } else {
        $list=get_translation_list($what, $lang, "$path$entry/", $list, "$rel_path$entry/");
      }
    }
    return($list);
  }

  // Builds the list of files for the selected options
  function build_translation_list($what, $lang) {
    global $CRM_path;
    $trans_list=get_translation_list($what, $lang, "$CRM_path/", $list, "");
    finalize_dump();
    return($trans_list);
  }

  // Deletes the specified file if it's older than the specified duration.
  // The default duration is two days.
  // The value returned is a little strange - it returns true if the file
  // has NOT been deleted and false if it has. This is done this way in order to
  // simplify the construct in build_translation_archive() below.

  // Please note that the mechanism implemented here doesn't prune all old
  // temp files - it merely makes sure the number of the temp files is limited
  // to the max of whatever you can produce in two days time, which typically
  // shouldn't be more than a couple, since temp files are deleted upon download anyway.
  function prune_temp($file,$dur=172800) {
    if ($ctime=@filectime($file)) {
      if ((time()-$ctime)>$dur) {
        return(!@unlink($file));
      } else {
        return(true);
      }
    } else {
      return(true);
    }
  }

  // Creates the lang_info.xml file and returns the full path to it
  function dump_lang_info() {
    global $OPT_lang_encodings, $temp_CRM_dir;
    $fname="$temp_CRM_dir/lang_info.xml";
    $fp=@fopen($fname,"w");
    if (!$fp) {
      return(false);
    }
    fputs($fp,"<languages>\n");
    while (list($id,$enc)=each($OPT_lang_encodings)) {
      fputs($fp,"  <lang>\n    <id>$id</id>\n    <encoding>$enc</encoding>\n  </lang>\n");
    }
    fputs($fp,"</languages>\n");
    fclose($fp);
    return($fname);
  }

  // Creates the temporary translations archive for the specified file list
  function build_translation_archive($list) {
    global $CRM_path, $temp_CRM_dir;
    if (!dump_lang_info()) {
      OPT_error(__("Unable to create lang_info.xml file!"));
      return(false);
    }
    reset($list["files"]);
    $i=0;
    while (is_file("$temp_CRM_dir/temp_OPT_lang$i.tar") && prune_temp("$temp_CRM_dir/temp_OPT_lang$i.tar")) {
      $i++;
    }
    while (is_file("$temp_CRM_dir/temp_OPT_lang$i.tar.gz") && prune_temp("$temp_CRM_dir/temp_OPT_lang$i.tar.gz")) {
      $i++;
    }
    $tar_index=$i;
    chdir($temp_CRM_dir);
    exec("tar -c --file=\"$temp_CRM_dir/temp_OPT_lang$tar_index.tar\" lang_info.xml 2>&1",$msg);
    if($msg) {
      $msg=implode("<BR>\n",$msg);
      OPT_error(sprintf(__("When trying to tar the file list, tar said: %s"),"<br>\n$msg"));
      @unlink("$temp_CRM_dir/temp_OPT_lang$tar_index.tar");
      return(false);
    }
    chdir($CRM_path);
    while (list($key, $val)=each($list["files"])) {
      exec("tar --append --file=\"$temp_CRM_dir/temp_OPT_lang$tar_index.tar\" \"$val\" 2>&1",$msg);
      if($msg) {
        $msg=implode("<BR>\n",$msg);
        OPT_error(sprintf(__("When trying to tar the file list, tar said: %s"),"<br>\n$msg"));
        @unlink("$temp_CRM_dir/temp_OPT_lang$tar_index.tar");
        return(false);
      }
    }
    exec("gzip $temp_CRM_dir/temp_OPT_lang$tar_index.tar 2>&1",$msg);
    if($msg) {
      $msg=implode("<BR>\n",$msg);
      OPT_error(sprintf(__("When trying to zip the tar file, gzip said: %s"),"<br>\n$msg"));
      @unlink("$temp_CRM_dir/temp_OPT_lang$tar_index.tar");
      @unlink("$temp_CRM_dir/temp_OPT_lang$tar_index.tar.gz");
      return(false);
    }
    return("$temp_CRM_dir/temp_OPT_lang$tar_index.tar.gz");
  }

  // Uploads the specified translation archive to the global OPT repository
  function upload_translation($file,$uname,$pwd) {
    $fp=fopen($file,"rb");
    $data[0][]="archive";
    // Need to encode binary data in a more reliable way than simply url encoding
    $data[0][]=imap_binary(fread($fp,filesize($file)));
    $data[1][]="uname";
    $data[1][]=$uname;
    $data[2][]="pwd";
    $data[2][]=$pwd;
    if (!($response=sendRequest("http://opt.lanifex.com/translate/upload.php",$data,"translate"))) {
      OPT_error(__("Unable to upload archive! Maybe your server's Internet connection isn't working?"));
      return(false);
    } elseif (!parseResponse($response,"translate")) {
      OPT_error(__("Uploaded but unable to parse server's response:<br>").nl2br($response));
      return(false);
    } else {
      global $translate_upload_confirm;
      if (!$translate_upload_confirm) {
        global $server_err;
        $server_err="<br>\n".$server_err;
        OPT_error(sprintf(__("Uploaded, parsed server's response, but the server answered this: %s"),$server_err));
        return(false);
      }
    }
    return(true);
  }

  // Checks if the specified folder contains translation subfolders
  function checkFolder($folder) {
    // echo("checking $folder - ");
    if((is_dir($folder)) && (is_dir("$folder/OPT_lang"))) {
      // echo("YES<br>\n");
      return(true);
    } else {
      // echo("no<br>\n");
      return(false);
    }
  }

  // This function strips language slashes.
  // In effect, it changes \" to ", then \\n to \n and finally \\ to \
  // It's the counterpart of addLangSlashes in this library.
  function stripLangSlashes($string) {
    $string=str_replace("\\\\","\\",str_replace("\\n","\n",str_replace("\\\"","\"",$string)));
    return($string);
  }

  // See stripLangSlashes
  function addLangSlashes($string) {
    $string=str_replace("\r","",str_replace("\"","\\\"",str_replace("\n","\\n",str_replace("\\","\\\\",$string))));
    return($string);
  }

  // Parses the specified translations file (PO file) and
  // returns an array containing the significant contents.
  function parseTrans($file) {
    $st=@file($file);
    // -2 because we get incremented once for the header and
    // once BEFORE actually starting to add data
    $idx=-2;
    while (list(,$line)=@each($st)) {
      $line=trim($line);
      if (!$line) {
        continue;
      } elseif (substr($line,0,1)=="#") {
        $result["comments"][$idx+1][]=trim(substr($line,1));
        continue;
      } elseif (substr($line,0,5)=="msgid") {
        $mode="id";
        $line=substr($line,strpos($line,'"'));
        $idx++;
      } elseif (substr($line,0,6)=="msgstr") {
        if ($mode!="head") {
          $mode="str";
        }
        $line=substr($line,strpos($line,'"'));
      }
      // echo("MODE: $mode; LINE: $line<br>\n");
      if (($mode=="id") && ($line=="\"\"")) {
        $mode="head";
        continue;
      }
      if ($mode=="id") {
        $result["content"][$idx]["id"].=stripLangSlashes(substr($line,1,-1));
      } elseif ($mode=="str") {
        $result["content"][$idx]["str"].=stripLangSlashes(substr($line,1,-1));
      } elseif ($mode=="head") {
        $line=stripLangSlashes(substr($line,1,-1));
        if (!$line) {
          continue;
        }
        $pos=strpos($line,":");
        $key=trim(substr($line,0,$pos));
        $val=trim(substr($line,$pos+1));
        $result["head"][$key]=$val;
      }
    }
    // var_dump($result);
    return($result);
  }

  // Matches two translation files and returns an array
  // of id's in the first array pointing to the respective id's
  // in the second.
  function matchTrans($trans, $ref) {
    while (list($key, $val)=@each($ref["content"])) {
      // We'll first assume the translation has the same indexes,
      // because most of the time, that will be the case:
      if ($trans["content"][$key]["id"]==$val["id"]) {
        $result["content"][$key]=$key;
      } else {
        // But if that's not the case, we'll have to search for the
        // respective ID before deciding it's not there
        for ($tkey=0;$tkey<sizeOf($trans["content"]);$tkey++) { //>
          $tval=$trans["content"][$tkey];
          // echo("Checking {$tval["id"]} against <b>{$val["id"]}</b>: ");
          if ($tval["id"]==$val["id"]) {
            // echo("YES");
            $result["content"][$key]=$tkey;
          }
          // else {
          //  echo("no");
          // }
          // echo("<br>\n");
        }
      }
    }
    // var_dump($result);
    return($result);
  }

  // Returns a string representing a level upwards from the specified file/directory
  // Expects $spec to be an absolute path/file specification.
  function getDir($spec) {
    $pos=strrpos($spec,"/");
    if (!$pos) {
      return("");
    }
    return(substr($spec,0,$pos));
  }

  // Creates the file if needed (along with the directory)
  // and writes the specified translations array to it.
  function writeTrans($trans, $transFile) {
    $fp=@fopen($transFile,"w");
    if (!$fp) {
      $lastDir=getDir($transFile);
      $tdir[]=$lastDir;
      while(($lastDir) && (!is_dir($lastDir))) {
        $lastDir=getDir($lastDir);
        $tdir[]=$lastDir;
      }
      for($i=sizeOf($tdir)-2;$i>-1;$i--) {
        if (!@mkdir($tdir[$i],0770)) {
          OPT_error(sprintf(__("Unable to create directory %s!"),$tdir[$i]));
          return(false);
        }
      }
      $fp=@fopen($transFile,"w");
      if (!$fp) {
        OPT_error(sprintf(__("Unable to write to file %s!"),$transFile));
        return(false);
      }
    }
    fputs($fp,"msgid \"\"\n");
    fputs($fp,"msgstr \"\"\n");
    while(list($key,$val)=each($trans["head"])) {
      if ($key) {
        fputs($fp,"\"".$key.": ".$val."\"\n");
      }
    }
    fputs($fp,"\n");
    while(list($key,$val)=each($trans["content"])) {
      for ($i=0;$i<sizeOf($trans["comments"][$key]);$i++) { //>
        if (!$trans["comments"][$key][$i]) {
          continue;
        }
        fputs($fp,"# ".$trans["comments"][$key][$i]."\n");
      }
      fputs($fp,"msgid ");
      fputs($fp,"\"".addLangSlashes($val["id"])."\"\n");
      fputs($fp,"msgstr ");
      fputs($fp,"\"".addLangSlashes($val["str"])."\"\n\n");
    }
    fclose($fp);
  }

?>
