<?
#
# Generates comma-separated (spreadsheet) report for downloading into a spreadsheet
#

$CRM_noheaders=true;
include "../../../include/OPT_lib.php";
auth_open();
//header("Content-Type: text/html");

// Get this just once per page load
$reasons = explode("|", get_setting("timesheet_nonproj_reasons"));
    
// Ensure that we have both a start and end date and groups
if (!$start_date || !$end_date || !$groups)
{
   OPT_error("Please provide a valid start and end date as well as a set of groups.");
   exit();
}

// Save the form submission parameters with a single query
crm_run_query("delete from user_settings where person = $CRM_user and type='timesheet_summary'");
$sql =  "insert into user_settings (person, type, name, value) values ";
$sql .= "($CRM_user, 'timesheet_summary', 'start_date', '".OPT_gpc_slashes($start_date)."'),";
$sql .= "($CRM_user, 'timesheet_summary', 'end_date', '".OPT_gpc_slashes($end_date)."')";
if (sizeOf($groups))
{
    $sql .= ",($CRM_user, 'timesheet_summary', 'groups_list', '".implode(",", $groups)."')";
}
CRM_run_query($sql);  // save user's preferences...

// Now get all the people to include in this report
$sql="select id from people where company in (".implode(",", $groups).") and (status & 1) and (status & ". ADMIN .") order by fname";
$people_result = CRM_run_query($sql);
while ($people_row = OPT_fetch_row($people_result))
{
    $people_ids[] = $people_row[0];

    // Calc this person's project hours (keyed to project's cost code)
    calcHoursPerCostCode($start_date, $end_date, $people_row[0]);

    // Calc this person's non-project hours
    calcHoursPerReason($start_date, $end_date, $people_row[0]);
}

// Output the header row
$row[] = "Name";
$proj_result=crm_elements("projects","order by name");
while ($proj_row=OPT_fetch_row($proj_result))
{
    $proj_ids[] = $proj_row[0];
}
$sql = "select distinct(cost_code) from projects where id in (".implode(",", $proj_ids).") order by cost_code";
$code_result = CRM_run_query($sql);
while ($code_row=OPT_fetch_row($code_result))
{
    $proj_cost_codes[] = $code_row[0];
}
for ($i=0; $i < sizeOf($proj_cost_codes); $i++)
{
    $row[] = $proj_cost_codes[$i];
}
for ($i=0; $i < sizeOf($reasons); $i++)
{
    $row[] = $reasons[$i];
}
$row[] = "Total";
$csvdata .= implode(",", $row) . "\r\n";   // end row


// Now write each row per person into a long string
//print_r($proj_cost_codes);
//echo("<br>");
for ($i=0; $i<sizeOf($people_ids); $i++)
{
    unset($total_for_person);
    unset($row);
    $row[] = getvalue("people", "CONCAT(fname,' ',lname)", $people_ids[$i]);
    for ($j=0; $j < sizeOf($proj_cost_codes); $j++)
    {
        //echo("Looking at hours_table[$proj_cost_codes[$j]][$people_ids[$i]] = " . $hours_table[$proj_cost_codes[$j]][$people_ids[$i]] . "<br>");
        $total_for_person += $hours_table[$proj_cost_codes[$j]][$people_ids[$i]];
        $row[] = sprintf("%.1f", $hours_table[$proj_cost_codes[$j]][$people_ids[$i]]);
    }

    for ($j=0; $j < sizeOf($reasons); $j++)
    {
        //echo("Looking at hours_table[$j][$people_ids[$i]] = " . $hours_table[$j][$people_ids[$i]] . "\r\n");
        $total_for_person += $reasons_table[$j][$people_ids[$i]];
        $row[] = sprintf("%.1f", $reasons_table[$j][$people_ids[$i]]);
    }

    // Add Totals ...
    $row[] = sprintf("%.1f", $total_for_person);
    $total_for_everybody += $total_for_person;
    $csvdata .= implode(",", $row) . "\r\n";
}

// Now output the project toal row on the bottom
unset($row);
$row[] = "Total";
for ($i=0; $i < sizeOf($proj_cost_codes); $i++)
{
    unset($total_for_project);
    for ($j=0; $j < sizeOf($people_ids); $j++)
    {
        $total_for_project += $hours_table[$proj_cost_codes[$i]][$people_ids[$j]];
    }
    $row[] = sprintf("%.1f", $total_for_project);
}
for ($i=0; $i < sizeOf($reasons); $i++)
{
    unset($total_for_reason);
    for ($j=0; $j < sizeOf($people_ids); $j++)
    {
        $total_for_reason += $reasons_table[$i][$people_ids[$j]];
    }
    $row[] = sprintf("%.1f", $total_for_reason);
}
$row[] = sprintf("%.1f", $total_for_everybody);
$csvdata .= implode(",", $row) . "\r\n";


// Now actually output the page content
$length=strlen($csvdata);
$filename="$end_date.csv";
header("Content-Type: application/vnd.ms-excel;");
header("Content-Disposition: attachment; filename=$filename");
header("Content-length: $length");
echo $csvdata;
auth_close();
?>





<? // Utility methods


// Show all the hours recorded for tasks on this day by this person
function calcHoursPerCostCode($start_date, $end_date, $person)
{
    global $hours_table, $proj_cost_codes;

    // Now get hours worked on tasks
    $sql="select p.cost_code, SUM(TIME_TO_SEC(tph.worked))/3600
          from task_progress_history as tph, task as t, projects as p
          where tph.taskID=t.id and tph.worked > 0 and p.id = t.ProjectID
                and tph.person=$person and tph.date >= '$start_date'
                and DATE_ADD('$end_date', INTERVAL 1 DAY) > tph.date
          group by p.cost_code";
    //echo("$sql<br/>");
    $result=CRM_run_query($sql);
    while ($row=OPT_fetch_row($result))
    {
        //echo("Adding $row[1] hours to $row[0]<br />");
        $hours[$row[0]] += $row[1];
    }
    
    // Also include any miscellaneous time for this project/person/day (reason = -1 for misc)
    $sql="select projects.cost_code, SUM(worked) from misc_time, projects where person=$person and day >= '$start_date'
          and projects.id = misc_time.project and day <  DATE_ADD('$end_date', INTERVAL 1 DAY)  
          and reason=-1 and project > 0 GROUP BY projects.cost_code";
    //echo("$sql<br/>");
    $misc_result=CRM_run_query($sql);
    while ($misc_row=OPT_fetch_row($misc_result))
    {
        //echo("Adding $misc_row[1] hours to $misc_row[0]<br />");
        $hours[$misc_row[0]] += $misc_row[1];
    }
 
    // Now loop through all projects this person has access to and store the cost codes
    $proj_result=crm_elements("projects","order by name");
    while ($proj_row=OPT_fetch_row($proj_result))
    {
        if (strlen($hours[$proj_row[4]]))
        {
            $hours_table[$proj_row[4]][$person] = $hours[$proj_row[4]];
            //echo("[$proj_row[4]][$person] = ".$hours_table[$proj_row[4]][$person]."</br>");
        }
    }
}

// Show all the hours recorded for non_proj time on this day by this person
function calcHoursPerReason($start_date, $end_date, $person)
{
    global $reasons_table, $reasons;

    for ($i=0; $i<sizeOf($reasons); $i++)
    {
        // First get sum of non proj time entered on timesheets
        $sql="select sum(worked) from misc_time where person = $person and '$start_date' <= day 
              and day < DATE_ADD('$end_date', INTERVAL 1 DAY) and reason = $i";
        $result = CRM_run_query($sql);
        $row = OPT_fetch_row($result);
        $reasons_table[$i][$person] = $row[0]; 
    }

    // We loop on each day from start to end calcing hours along the way
    $start = strtotime($start_date);
    $end = strtotime($end_date);
    for ($date=$start; $date < $end; $date+=86400)
    {
        $weekday = date("w", $date);
        if (($weekday != 0) && ($weekday != 6))
        {
            $sql="select reason, UNIX_TIMESTAMP(start_date), UNIX_TIMESTAMP(end_date)
                      from absences where person=$person and start_date <= 
                      DATE_ADD(FROM_UNIXTIME($date), INTERVAL 1 DAY) and end_date > FROM_UNIXTIME($date)  
                      and reason >=0 and count_on_timesheet = 1";
            $result=CRM_run_query($sql);
            while($row=OPT_fetch_row($result))
            {
                // Need to figure out how many hours to count
                $strdate=date("Y-m-d", $date);
                $noon = strtotime("$strdate 12:00:00");
                $onepm = strtotime("$strdate 13:00:00");
                $eightam = strtotime("$strdate 08:00:00");
                $fivepm = strtotime("$strdate 17:00:00");
                if ($row[1] < $eightam)
                {
                    $row[1] = $eightam;
                }
                if ($row[2] > $fivepm)
                {
                    $row[2] = $fivepm;
                }
                $secs=$row[2] - $row[1];
                
                if (($row[1] <= $noon) && ($row[2] > $onepm))
                {
                    $secs -= 60*60; // minus one hour for lunch
                }
                if ($secs > (8 * 60 *60))
                {
                        $secs = 8 * 60 * 60;
                }
                $reasons_table[$row[0]][$person] += ($secs/3600); // add on hours per reason 
            }
        }
    }
}

?>