<?
// Generates a filter string of SQL based on all the fields selected
// by the user during an advanced search

// ******************* String section **************************
if ($subject_string && $subject_string_type)
{
    $filter_components[] = "(" .get_string_search_filter($subject_string_type, $subject_string, "title") .")";
}

if ($body_string && $body_string_type)
{
    $filter_components[] = "(" .get_string_search_filter($body_string_type, $body_string, "body") .")";
}

if ($source_string && $source_string_type)
{
    $filter_components[] = "(" .get_string_search_filter($source_string_type, $source_string, "emailSrc") .")";
}

if ($forum_string && $forum_string_type)
{
    $forum_table_filter_for_title = get_string_search_filter($forum_string_type, $forum_string, "title");
    $forum_table_filter_for_content = get_string_search_filter($forum_string_type, $forum_string, "content");
    $sql = "select att_id from forums where att_type = 'email' and $forum_table_filter_for_title OR $forum_table_filter_for_content";
    $result = crm_run_query($sql);
    while ($row = OPT_fetch_row($result))
    {
        $forum_email_ids[] = $row[0];
    }
    if ($forum_email_ids)
    {
        $filter_components[] = "id IN (". implode(",", $forum_email_ids) .")";
    }
    else
    {
        $filter_components[] = "1=0";  // No forum entries met this criteria so no emails should be found
    }
}


// ******************* People section **************************
if (sizeOf($sender) && ($sender > -1))
{  
   unset($modifier);
   if (!$sender_modifier)
   {
      $modifier="NOT";
   }  
   $filter_components[] = "$modifier (src = ".abs($sender) .")";     
}

if ($email_forum_submitter > 0)
{
   unset($modifier);
   if (!$forum_submitter_modifier)
   {
      $modifier="NOT";
   }
   $sql="select att_id from forums where att_type= 'email' and (src = ".abs($email_forum_submitter).")";
   $result=CRM_run_query($sql);
   while ($row=OPT_fetch_row($result))
   {
      $forum_submitter_email_ids[] = $row[0];
   }
   if ($forum_submitter_email_ids)
   {
        $filter_components[] = "id $modifier IN (". implode(",", $forum_submitter_email_ids) .")";
   }
   else
   {
        $filter_components[] = "$modifier 1=0";  // No emails submitted (or not) by this individual
   }
}


// ******************* Dates and Attachments section **************************
if (strlen($sent_date_start) && (strlen($sent_date_start) &&
   (strtotime($sent_date_start) > 0) && (strtotime($sent_date_end) > 0)))
{
   $filter_components[] = "(date > '$sent_date_start' AND date < DATE_ADD('$sent_date_end', INTERVAL 1 DAY))";
}

if (strlen(trim($filename_string)))
{
   $safe_name = OPT_gpc_slashes(trim($filename_string));
   $sql = "select e_mail from e_attach where fname = '$safe_name'";
   $result=CRM_run_query($sql);
   while ($row=OPT_fetch_row($result))
   {
      $filename_email_ids[] = $row[0];
   }
   if ($filename_email_ids)
   {
        $filter_components[] = "id IN (". implode(",", $filename_email_ids) .")";
   }
   else
   {
        $filter_components[] = "1=0";  // No emails submitted (or not) by this individual
   }

}


// Fake out if no criteria selected at all; also means that completely empty form fields mean include all!
if ($filter_components)
{
   // Need a leading AND as first clause limits based on 'aproj = $proj' automatically...
   $filter = "AND " . implode(" AND ", $filter_components);
}





// Generates the filter from a string search on title or body etc...
function get_string_search_filter($type, $text, $field)
{
    // Handle the particular search type requested
    if (0 == strlen($text))
    { 
        $filter = "1 = 0";  // No text provided
    }
    else if ($type == "substring")
    {
        $substring=OPT_gpc_slashes($text);
        $filter = "$field like '%$substring%'";
    }
    else if ($type == "allwords")
    {
        $words=explode(" ", $text);
        if (sizeOf($words) > 0)
        {
            for ($i=0; $i<sizeOf($words); $i++)
            {
                $word = OPT_gpc_slashes($words[$i]);
                $clauses[] =  "$field like '%$word%'";
            }
            $filter = $filter . implode(" and ", $clauses);
        }
        else
        {  // just spaces provided; they get no results
            $filter = $filter ." 1 = 0";
        }
    }
    else if ($type == "anywords")
    {
        $words=explode(" ", $text);
        if (sizeOf($words) > 0)
        {
            for ($i=0; $i<sizeOf($words); $i++)
            {
                $word = OPT_gpc_slashes($words[$i]);
                $clauses[] =  "$field like '%$word%'";
            }
            $filter = implode(" or ", $clauses);
        }
        else
        {   // just spaces provided; they get no results
            $filter = " 1 = 0";
        }
    }
    else if ($type == "regexp")
    {
        $expr=OPT_gpc_slashes($text);
        $filter = "$field regexp '$expr'";
    }
    else if ($type == "notregexp")
    {
        $expr=OPT_gpc_slashes($text);
        $filter = "$field not regexp '$expr'";
    }
    return $filter;
}


?>