<?
#
# Custom authentication library - native OPT
#
# Bogdan Stancescu <bogdan@lanifex.com>, August 2002
# Revision 1.0

  // These two functions run at the beginning and end of every page in OPT.
  // They run the phplib authentication.

  /*
    If you want to customize the authentication mechanism, please note the following:

    1. $CRM_guest should be set to evaluate true for users with guest access and false
       otherwise. This is currently done via Check_IP() but you may wish to change that;

    2. This should be the place to set $CRM_user and $CRM_comp;

    3. If the user is not authenticated, custom_page_open() below MUST provide the login page;
       custom_page_open() MUST also exit if needed (i.e. after displaying the login page);

    4. custom_page_open() below MUST set the following global variables:
       $CRM_user - this MUST be a valid ID from the people database in OPT. The value 0 (zero)
         MUST NOT be returned under any circumstances - if the user is not authenticated the
         function must perform whatever action and halt execution (using exit()).
         $CRM_user may also be set to the string value of "nobody" if the user is a guest.
       $CRM_guest - if the user is determined to be a guest (typically using Check_IP() which
         is implemented in include/auth.php) then this variable must be set to evaluate
         true and $CRM_user MUST be set to "nobody";

    5. The first five user ID's are reserved for special users, this way:
       1 - admin demo user;
       2 - regular demo user;
       3-5 - reserved for future development.
       If you return a value of 1 or 2, OPT will manage demo user access based on the current
       settings (see include/OPT_lib.php).

    6. Since you will typically implement a completely distinct authentication mechanism
       (i.e. which has nothing to do with OPT's people table), you MUST implement a mechanism
       to also synchronize the data from the external source with OPT's people table data.
       This document does not specify any specifics on how to implement that because of the
       wide variety of external authentication systems, but you will necessarily have to make
       sure you synchronize the two sources using a clean, secure and reliable method.
       (e.g. NOT duplicating users in OPT's people table).

       One suggestion on implementation would be to match the username/MD5(password) pair
       against username/password pairs in OPT. If there is one matching entry, all applicable
       data from the external source MUST be updated in OPT's people table. If no matching
       entry is found, your code MUST retrieve any applicable data which identifies the person
       who logged in from the external source (typically the first name/last name pair)
       and check that against OPT's people table. If one entry is found, all applicable data
       from the external source MUST be updated in OPT's people table. If more than one entry
       is found, your code MUST match some other relevant data if available (such as mobile
       phone number or e-mail address) or not allow logging in and notifying the administrator
       at $CRM_email (recommended). If you choose to implement the first behaviour an exact
       (or at most case-insensitive) match against NON-EMPTY values must be found before
       allowing login - otherwise you open security holes. The code MUST check that the value
       for all matching users is non-empty in OPT and MUST check if the values are distinct
       (i.e. it MUST NOT check that one user matches and use that regardless of whether the
       other users have the same data or not).

    7. Please use the global variable $_OPT["auth"]["loaded"] as seen here to emulate
       include_once() - we have to be PHP 3.0.6 compliant and include_once() wasn't
       implemented.

  */

  global $_OPT;
  if (!$_OPT["auth"]["loaded"]) {
    function custom_page_open()
    {
      global $sess, $auth, $perm, $CRM_guest, $CRM_user, $request_login;

      OPT_page_open(array( "sess" => "OPT_Session",
                      "auth" => "OPT_Auth",
                      "perm" => "OPT_Perm"
                    )
              );

      if (!eregi("candidate",$REQUEST_URI)&&
          (!$perm->have_perm("admin"))
        ) {
        $auth->login_if(($request_login) || (!($CRM_guest=Check_IP())));
      } else {
        $CRM_guest=false;
      }
      $CRM_user=$auth->is_authenticated();
    }

    function custom_page_close()
    {
      global $CRM_authentify, $CRM_guest;
      if (! ($CRM_authentify || $CRM_guest)) {
        OPT_page_close();
      }
    }

    $_OPT["auth"]["loaded"]=true;

    function custom_logout()
    {
      global $auth;
      $auth->logout();
      // Although this typically simply runs custom_page_close() above,
      // please leave it here instead of custom_page_close() unless you
      // don't want auth_page_close() to run at this point.
      auth_close();
    }
  }

?>
