<?
#
# Commonly used methods for getting lists of things
#


#
# Returns a result set of all the people involved with a project
# proj: The project id number
# people_fields: The fields from the people table to return in the result
# role: The roles to restrict to (DEVELOPER or CUSTOMER)
#
function get_people_involved_with_project($proj, $people_fields="id, fname, lname", 
                                          $role=DEVELOPER, $order_by="order by fname")
{
    global $CRM_demo;
    
    if ($role == DEVELOPER)
    {
        $role_sql="and role = 'Developer'";
    }
    elseif ($role == CUSTOMER)
    {
        $role_sql="and role = 'Customer'";
    }
    else
    {
        $role_sql="";
    }

    if (!$CRM_demo)
    {
        $demo_sql = "and people.id > 5";
    }

    // Get list of developers that are in groups with access to this project
    $sql="select company from company2project, companies where project=$proj $role_sql and company=id";
    $comp_result=CRM_run_query($sql);
    while ($comp_row=OPT_fetch_row($comp_result))
    {
        $sql="select id from people where company=$comp_row[0] and (status&1) = 1 $demo_sql";
        $comp_user_result=CRM_run_query($sql);
        while ($comp_user_row=OPT_fetch_row($comp_user_result))
        {
            if (!@in_array($comp_user_row[0],$people))
            {
                $people[] = $comp_user_row[0];
            }
        }
    }
    
    // Get list of individuals with developer access to this project
    $sql="select id from people_proj, people where project=$proj and person = id $role_sql";
    $dev_result=CRM_run_query($sql);
    while ($dev_row=OPT_fetch_row($dev_result))
    {
        if (!@in_array($dev_row[0],$people))
        {
            $people[] = $dev_row[0];
        }
    }
    
    // Get a sorted list of names based on the array of people's id numbers
    $sql="select $people_fields from people where (status&1) = 1 and id IN (";
    for ($i=0; $i<sizeOf($people); $i++)
    {
        $sql=$sql.$people[$i].', ';
    }
    $sql=$sql."-666) $order_by";  // tack on a fake id number to finish out commas
    $result=CRM_run_query($sql);
    return $result;
}


#
# Returns the nearest (next) milestone coming up for a project
#   - result row of [id, name] returned or 0 if none found
#
function get_next_milestone($proj)
{
    $sql="select id, name from project_milestones where project=$proj and 
          DATE_ADD(due_date, INTERVAL 1 DAY) > NOW() order by due_date LIMIT 1"; 
    $result=CRM_run_query($sql);
    if (OPT_num_rows($result) == 1)
    {
        return OPT_fetch_row($result);
    }
    else
    {
        return 0;  // couldn't find one
    }
}


// Gets a result set of id, full name of people who are developers on projects managed by this person
function get_all_developers_under_PM($person, $people_fields="people.id, people.fname, people.lname", 
         $order_by="order by fname")
{
    global $CRM_demo;
    if (!$CRM_demo)
    {
        $demo_sql = "and people.id > 5";
    }

    // Get a list of projects this person manages
    $sql="select project from project_managers where manager = $person";
    $result = CRM_run_query($sql);
    while ($row=OPT_fetch_row($result))
    {
        $proj_ids[] = $row[0];
    }
    
    // Get list of developers that are in groups with access to this project
    $sql="select company from company2project, companies where project in (".implode(",", $proj_ids).")
          and role='Developer' and company=id and status > 0";
    $comp_result=CRM_run_query($sql);
    while ($comp_row=OPT_fetch_row($comp_result))
    {
        $sql="select id from people where company=$comp_row[0] and (status&1) = 1 $demo_sql";
        $comp_user_result=CRM_run_query($sql);
        while ($comp_user_row=OPT_fetch_row($comp_user_result))
        {
            if (!@in_array($comp_user_row[0],$people))
            {
                $people[] = $comp_user_row[0];
            }
        }
    }

    // Get list of individuals with developer access to this project
    $sql="select id from people_proj, people where project in (".implode(",", $proj_ids).") 
          and person = id and role='Developer' and (status&1) = 1";
    $dev_result=CRM_run_query($sql);
    while ($dev_row=OPT_fetch_row($dev_result))
    {
        if (!@in_array($dev_row[0],$people))
        {
            $people[] = $dev_row[0];
        }
    }

    // Get a sorted list of names based on the array of people's id numbers
    if ( sizeOf($people) > 0)
    {
        $sql="select id, $people_fields from people where id IN (";
        for ($i=0; $i<sizeOf($people); $i++)
        {
            $sql=$sql.$people[$i].', ';
        }
        $sql=$sql."-666) $demo_sql $order_by";  // tack on a fake id number to finish out commas
        $people_names_result=CRM_run_query($sql);
    }
    return $people_names_result;
}


// Missing in PHP < version 4.2
function myfloatval( $strValue )
{
    $floatValue = preg_replace( "/[^0-9\.]*/", "", $strValue );
    return $floatValue;
}



// Returns a result set of users that are overseen by the current user (CRM_user)
// Overseen by an admin, manager or payroll contact that is
function get_people_list_for_supervisor($people_fields="people.id, people.fname, people.lname", $order_by="order by fname")
{
    global $CRM_user, $CRM_demo;
    if (!$CRM_demo)
    {
        $demo_sql = "and people.id > 5";
    }

    if (is_admin())
    { // Admins can see all people
        $sql="select $people_fields from people 
              where (people.status & 1)=1 $demo_sql
              $order_by";
        return crm_run_query($sql);
    }
    else if (person_is_payroll_contact($CRM_user))
    {   
        $sql="select $people_fields from people left join companies on people.company = companies.id 
              where (people.status & 1)=1 and ((companies.payroll_contact = $CRM_user) OR (people.id = $CRM_user)) $demo_sql
              order by people.fname";
        return CRM_run_query($sql);
    }
    else if (person_is_project_manager($CRM_user))
    {   
        return get_all_developers_under_PM($CRM_user);
    }
}
  
// Returns an array of project ids which this person has "developer" access to
function get_projects_with_developer_access($person)
{
    // First get all projects their group has dev access to
    $user_group = getvalue("people", "company", $person);
    $sql="select distinct(id) from projects as pr, company2project as c2p 
        where c2p.company=$user_group and c2p.project = pr.id 
        and c2p.role='Developer' and pr.active=1 order by pr.name";
    $result=CRM_run_query($sql);
    while ($row = OPT_fetch_row($result))
    {
        $proj_ids[] = $row[0];
    }
    
   // Now get any projects that they are individually associated with
    $sql="select distinct(id) from people_proj as pp, projects as pr
        where pp.person=$person
        and pr.id=pp.project and pp.role='Developer' and pr.active=1 order by pr.name";
    $result=CRM_run_query($sql);
    while ($row = OPT_fetch_row($result))
    {
        $proj_ids[] = $row[0];
    }

    return $proj_ids;
}
    
// Returns an array of project ids which this person has "customer" access to
function get_projects_with_customer_access($person)
{
    // First get all projects their group has dev access to
    $user_group = getvalue("people", "company", $person);
    $sql="select distinct(id) from projects as pr, company2project as c2p 
        where c2p.company=$user_group and c2p.project = pr.id 
        and c2p.role='Customer' and pr.active=1 order by pr.name";
    $result=CRM_run_query($sql);
    while ($row = OPT_fetch_row($result))
    {
        $proj_ids[] = $row[0];
    }
    
   // Now get any projects that they are individually associated with
    $sql="select distinct(id) from people_proj as pp, projects as pr
        where pp.person=$person
        and pr.id=pp.project and pp.role='Customer' and pr.active=1 order by pr.name";
    $result=CRM_run_query($sql);
    while ($row = OPT_fetch_row($result))
    {
        $proj_ids[] = $row[0];
    }

    return $proj_ids;
}
?>