<?
// GDD: Broke apart element permission in urights.php into a modular layout for ease of changing


// ******** First set of methods are called from crm_perm() in urights.php  ************
// Find out if a specific element in the table 
// is accessible to the current user
// if returns 0, not allowed
// 1 : all rights
// 2 : edit but not delete
// 3 : view only


function elem_perm_for_people($table_id)
{
    global $CRM_user,$CRM_comp, $CRM_guest;

    // Some sites may have enabled group hiding for all but privileged first group=1
    $hide_groups = get_setting("hide_other_groups_from_all_but_first_group");

    // If current user is the manager of user's group they can edit them       
    $user_group = getvalue("people", "company", $table_id);
    $group_contact = getvalue("companies", "contact", $user_group);
    if (($table_id==$CRM_user) || ($group_contact == $CRM_user)) 
    {   // can edit themselves
        $allow=2;
    } 
    else if ($hide_groups && ($CRM_group != 1) && ($user_group != $CRM_comp))
    {
        $allow=0; // Not in privileged first group, hide groups enabled and user in different group
    }
    else
    {
        $allow=3; // VIEW access granted
    }
    return $allow;
}

function elem_perm_for_companies($table_id)
{
    global $CRM_user,$CRM_comp, $CRM_guest;

    // Some sites may have enabled group hiding for all but privileged first group=1
    $hide_groups = get_setting("hide_other_groups_from_all_but_first_group");

    // Also if the person is the primary contact for their group, they can EDIT
    $primary_contact = getvalue("companies", "contact", $table_id);

    if (($table_id==$CRM_comp) && ($primary_contact == $CRM_user))
    {
        $allow=2;  // Primary contact of group can EDIT
    }
    else if ($hide_groups && ($CRM_comp != 1) && ($table_id != 1) && ($table_id != $CRM_comp)) 
    {
        $allow=0;  // user isn't in this group and site hides other groups
    }   
    else
    {
        $allow=3;  // EDIT access granted
    }   
    return $allow;
}

// Note: often the project permissions are a fallback for elements containted within it
function elem_perm_for_projects($table_id)
{
    global $CRM_user,$CRM_comp, $CRM_guest;

    // Full access for system admins
    if (is_admin())
    {
        return 1;  // full access to this project
    }

    // Project managers
    if  (is_project_manager($table_id))
    {
        return 2;  // edit access to this project
    }

    // Are they in a group with access (either developer or customer)?
    $sql="select count(*) from company2project where company = $CRM_comp and project = $table_id";
    $result=CRM_run_query($sql);
    $row=OPT_fetch_row($result);
    if ($row && ($row[0] > 0))
    {
        return 3;  // view access to this project
    }
    
    // Are they listed as an individual with access to this project (either developer or customer)
    $sql="select count(*) from people_proj where person= $CRM_user and project = $table_id";
    $result=CRM_run_query($sql);
    $row=OPT_fetch_row($result);
    if ($row && ($row[0] > 0))
    {
        return 3;  // view access to this project
    }
    
    return 0; // no access
}

function elem_perm_for_documents($table_id)
{
    global $CRM_user,$CRM_comp, $CRM_guest;
    $field = "(src=$CRM_user)";
    $sql="select id from documents where $field and id=$table_id";
    $result=CRM_run_query($sql);
    if ($myrow = OPT_fetch_array($result)) 
    {
        // user is owner or creator
        $allow = 1;
    } 
    elseif ($proj=abs(getvalue("documents","aproj",$table_id)))
    {
        //check if user has view/edit permission on the project
        if (is_project_manager($proj))
        {
           $allow = 1;  // PM's can delete any doc in their project
        }
        else
        {       
           $allow = elem_perm_for_projects($proj);
        }
    }
    return $allow;
}


function elem_perm_for_requests($table_id)
{
    global $CRM_user,$CRM_comp, $CRM_guest;
    $proj=abs(getvalue("requests","aproj", $table_id));
    $owner=abs(getvalue("requests", "owner", $table_id)); 

    // Did this project manager tighten down access
    $only_pm_can_edit_requests = get_project_setting($proj, "only_pm_can_edit_requests");
    if ($only_pm_can_edit_requests == "1")
    {
        // check if user has view permission on the project
        $allow = elem_perm_for_projects($proj);
    } 
    else 
    {   // PM has liberal policy, allow user to edit
        if ($owner == $CRM_user)
        {
            $allow=2;
        }
        else
        {
            $allow = elem_perm_for_projects($proj);
        }
    }

    // Increase permissions if this is the project manager
    if (is_project_manager($proj)) 
    {
        // PM should be able to delete, view and edit requests
        $allow = 1;
    } 

    return $allow;
}


function elem_perm_for_email($table_id)
{
    global $CRM_user,$CRM_comp, $CRM_guest;
    if ($proj=abs(getvalue("email", "project",$table_id))) 
    {
        if (is_project_manager($proj)) 
        {   
           // PM should be able to delete, view and edit requests
           $allow = 1;
        } 
        else
        {
           $allow=crm_perm("projects",$proj);
        }
    }
    return $allow;
}


function elem_perm_for_task($table_id)
{
    global $CRM_user,$CRM_comp, $CRM_guest; 
    // checking if user is owner of project or
    // just developer 
    $sql="select owner from person_task where
          personID = $CRM_user and taskID = $table_id";
    $result=CRM_run_query($sql);
    if ($myrow = OPT_fetch_array($result))
    {
        if ($myrow['owner']=="Y")
        {
            $allow=1; // full privileges
        }
        elseif ($myrow['owner']=="N")
        {
            $allow=2; // edit privileges for those involved, but not owner
        }
    }
    elseif ($proj=abs(getvalue("task", "projectID",$table_id))) 
    {
        // Then they get the privileges for the project
        $allow = crm_perm("projects",$proj);
    }
    return $allow;
}


function elem_perm_for_meeting($table_id)
{
    global $CRM_user,$CRM_comp, $CRM_guest;

    if (get_setting("allow_all_participants_to_edit_meetings"))
    {
        // Allow access for people involved in the meeting
        $sql = "select person from meeting2person where meeting=$table_id
                and person=$CRM_user";
        $result=CRM_run_query($sql);
        if (OPT_fetch_row($result))
        {
            $allow=2;
        } 
    }

    if  ($CRM_user == getvalue("meeting", "creator", $table_id))
    {
        $allow=1;
    }

    if (!$allow) 
    {
        // Also allow access for people who have access to any projects
        $sql = "select project from meeting2project where meeting=$table_id";
        $result = CRM_run_query($sql);
        while ($allow==0 && ($rec=OPT_fetch_array($result)))
        {
            $proj = $rec["project"];
            $allow=elem_perm_for_projects($proj);
        }
    } 
    return $allow;
}

function elem_perm_for_events($table_id)
{
    global $CRM_user,$CRM_comp, $CRM_guest;
    // Allow access EDIT access to submitter and PM
    $src=getvalue("events", "src", $table_id);
    $project=getvalue("events", "project", $table_id);
    if ($src == $CRM_user)
    {
        $allow=1;
    } 
    else if ($proj && is_project_manager($proj))
    {
        $allow=1;
    }
    else if ($proj)
    {   // Not event submitter or project manager
        $allow=elem_perm_for_projects($proj);
    } 
    else
    {   // Global event 
        $allow=3;  // VIEW only for all
    }
    return $allow;
}

function elem_perm_for_absences($table_id)
{
    global $CRM_user,$CRM_comp, $CRM_guest;
    $person=getvalue("absences", "person", $table_id);
    if ($person == $CRM_user)
    {
        $allow=1;
    } 
    else
    {   // Not event submitter or project manager
        $allow=elem_perm_for_people($person);
    } 
    return $allow;
}

function elem_perm_for_code_repositories($table_id)
{
    global $CRM_user,$CRM_comp, $CRM_guest;
    $project=getvalue("code_repositories", "project", $table_id);
    if (crm_project_participant($CRM_user, $project, DEVELOPER))
    {
        $allow = 3;  // Team members have VIEW privilege
    }
    if (is_project_manager($project))
    {
        $allow = 1;  // Can EDIT or DELETE as well
    }
    return $allow;
}

//***************** End of helper methods for crm_perm() in urights.php ****************


//***************** Beginning of helper methods for crm_elements() in urights.php ******
function elems_for_admin($table, $extra_sql)
{
    global $CRM_user, $CRM_comp, $proj, $CRM_guest, $CRM_demo;
    if (!in_array($table,array("documents","requests","email","people","companies"))) {
        $sql="select * from $table where 1=1 $extra_sql";
    } 
    else 
    {
        if (($table=="documents") || ($table=="requests")) 
        {
            if ($proj) 
            {
                $sql="select * from $table where aproj=$proj $extra_sql";
            } 
            else
            {
                // It means we're only counting -- hence the syntax
                $sql="select * from $table where 1=1 $extra_sql";
            }
        } 
        elseif ($table=="people") 
        {
            if (!$CRM_demo)
            {
                $demo_sql = " and people.id > 5";   
            }
            $sql="select * from people where (status & 1)=1 $demo_sql $extra_sql";
            $result=CRM_run_query($sql);
        } 
        elseif ($table=="companies") 
        {
            $sql="select * from companies where status=1 $extra_sql";
            $result=CRM_run_query($sql);
        } 
        elseif ($table=="email")
        {
            if ($proj) 
            {
                $sql="select * from email where project=$proj $extra_sql";
            }
            else 
            {
                // It means we're only counting -- hence the syntax
                $sql="select * from email where $extra_sql";
            }
        }
    }
    $result=CRM_run_query($sql);
    return $result;
}


function elems_for_people($table, $extra_sql)
{
    global $CRM_user, $CRM_comp, $proj, $CRM_guest, $CRM_demo;

    // Some sites may have enabled group hiding for all but privileged first group=1
    $hide_groups = get_setting("hide_other_groups_from_all_but_first_group");

    if (!$CRM_demo)
    {   // Used to exlude any demo users from being seen outside demo mode
        $demo_sql = " and people.id > 5";
    }

    if ($hide_groups && ($CRM_comp != 1))
    {
       $sql="select * from people where ((status & 1) = 1) and  company in (1, $CRM_comp) $demo_sql $extra_sql";
    }
    else
    {
       $sql="select * from people where ((status & 1) = 1) $demo_sql $extra_sql";
    }   
    $result=crm_run_query($sql);
    return $result;
}

function elems_for_companies($table, $extra_sql)
{
    global $CRM_user, $CRM_comp, $proj, $CRM_guest;
    // Some sites may have enabled group hiding for all but privileged first group=1
    $hide_groups = get_setting("hide_other_groups_from_all_but_first_group");

    if ($hide_groups && ($CRM_comp != 1))
    {  
        $sql="select * from companies where id in (1, $CRM_comp) && status=1 $extra_sql";
    }
    else
    {
        $sql="select * from companies where status=1 $extra_sql";
    }
    $result=CRM_run_query($sql);
    return $result;
}

function elems_for_projects($table, $extra_sql)
{
    global $CRM_user, $CRM_comp, $proj, $CRM_guest;

    // Select all projects where they are project manager
    $sql="select projects.id from projects left join project_managers on projects.id = project_managers.project
          where project_managers.manager = $CRM_user";
    $result=CRM_run_query($sql);
    while ($myrow=OPT_fetch_array($result)) {
        $proj_ids[] = $myrow[0];
    }

    // Select all projects for which their group has access
    $sql="select project from company2project where company=$CRM_comp";
    $result=CRM_run_query($sql);
    while ($myrow=OPT_fetch_array($result)) {
        $proj_ids[] = $myrow[0];
    }

    // Select all projects where they have individual access
    $sql="select project from people_proj where person=$CRM_user";
    $result=CRM_run_query($sql);
    while ($myrow=OPT_fetch_array($result)) {
        $proj_ids[] = $myrow[0];
    }
    
    $cond="id IN (";
    for ($i=0; $i<sizeOf($proj_ids); $i++)
    {
        $cond=$cond."$proj_ids[$i], ";
    }
    $cond=$cond." -666)";
   
    $sql="select * from projects where $cond and active = 1 $extra_sql";
    $result=CRM_run_query($sql);
    return $result;
}

function elems_for_documents($table, $extra_sql)
{
    global $CRM_user, $CRM_comp, $proj, $CRM_guest;
    if (! $proj) {
        // If no project is set then we'll have to browse through all of them.
        $result=elems_for_projects($table, "");
        while ($myrow=OPT_fetch_row($result)) {
                    // now let's see what documents are in there
            $projs.="aproj=$myrow[0] or ";
        }
        $projs=substr($projs,0,-4); // Trimming the last " or "
        if ($projs) {
            $sql="select * from $table where ($projs) $extra_sql";
        } else {
            // Once again, the ugly method of returning an empty handle...
            $sql="select * from $table where 1=0";
        }
        $result=CRM_run_query($sql);
    } else {
        // If a project is set we only need to look inside.
        // We must first make sure the user has access to the project...
        if (crm_perm("projects",$proj)) {
            $sql="select * from $table where aproj=$proj $extra_sql";
        } else {
            // Yet again, the ugly method of returning an empty handle...
            $sql="select * from $table where 1=0";
        }
        $result=CRM_run_query($sql);
    }
    return $result;
}


function elems_for_requests($table, $extra_sql)
{
    global $CRM_user, $CRM_comp, $proj, $CRM_guest;
    if (! $proj) {
        // If no project is set then we'll have to browse through all of them.
        $result=elems_for_projects($table);
        while ($myrow=OPT_fetch_row($result)) {
                    // now let's see what documents are in there
            $projs.="aproj=$myrow[0] or ";
        }
        $projs=substr($projs,0,-4); // Trimming the last " or "
        if ($projs) {
            $sql="select * from $table where ($projs) $extra_sql";
        } else {
            // Once again, the ugly method of returning an empty handle...
            $sql="select * from $table where 1=0";
        }
        $result=CRM_run_query($sql);
    } else {
        // If a project is set we only need to look inside.
        // We must first make sure the user has access to the project...
        if (crm_perm("projects",$proj)) {
            $sql="select * from $table where aproj=$proj $extra_sql";
        } else {
            // Yet again, the ugly method of returning an empty handle...
            $sql="select * from $table where 1=0";
        }
        $result=CRM_run_query($sql);
    }
    return $result;     
}


function elems_for_email($table, $extra_sql)
{
    global $CRM_user, $CRM_comp, $proj, $CRM_guest;
    if (! $proj) {
        // If no project is set then we'll have to browse through all of them.
        $result=elems_for_projects($table);
        while ($myrow=OPT_fetch_row($result)) {
           // now let's see what e-mails are in there
           $projs.="project=$myrow[0] or ";
        }
        $projs=substr($projs,0,-4);
        if ($projs) {
          $sql="select * from email where ($projs) $extra_sql";
        } else {
          $sql="select * from email where project=-1";
        }
        $result=CRM_run_query($sql);
     } else {
        // If a project is set we only need to look inside.
        // We must first make sure the user has access to the project...
        if (crm_perm("projects",$proj)) {
           $sql="select * from email where project=$proj $extra_sql";
        } else {
           // Ok, I know this ain't elegant but what can I do?
           $sql="select * from email where project=-1";
        }
     }
    $result=CRM_run_query($sql);
    return $result;     
}

function elems_for_tasks($table, $extra_sql)
{
    global $CRM_user, $CRM_comp, $proj, $CRM_guest;
    if (! $proj) {
        // If no project is set then we'll have to browse through all of them.
        $result=elems_for_projects($table);
        while ($myrow=OPT_fetch_row($result)) {
           // now let's see what e-mails are in there
           $projs.="ProjectID=$myrow[0] or ";
        }
        $projs=substr($projs,0,-4);
        if ($projs) {
          $sql="select * from task where ($projs) $extra_sql";
        } else {
          $sql="select * from task where ProjectID=-1";
        }
        $result=CRM_run_query($sql);
     } else {
        // If a project is set we only need to look inside.
        // We must first make sure the user has access to the project...
        if (crm_perm("projects",$proj)) {
           $sql="select * from task where ProjectID=$proj $extra_sql";
        } else {
           // Ok, I know this ain't elegant but what can I do?
           $sql="select * from task where ProjectID=-1";
        }
     }
    $result=CRM_run_query($sql);
    return $result;     
}

//***************** End of helper methods for crm_elements() in urights.php ******

?>
