<?
#
# General use library
#
# All functions
# Bogdan Stancescu <bogdan@lanifex.com>, November 2001
# unless specified otherwise
# Revision 1.0

  // =====================================================================
  //
  //                   Really generic functions
  //
  // =====================================================================

  if (!function_exists("isphpver"))
  {
    // Function to determine if the current version of PHP is
    // at least the one specified as a string in the parameter.
    // This is needed because string comparison isn't always accurate:
    // "4.0.2">"4.0.12" as strings, although that's not true when
    // speaking of comparing versions.

    // Usage example:
    // if (!isphpver("4.0.2"))
    // {
    //   [do something]
    // }

    function isphpver($minver)
    {
      $itis=2; // That is, undecided
      $minimum=explode(".",$minver);
      $current=explode(".",phpversion());
      for ($i=0;(($i<sizeof($current)) && ($i<sizeof($minimum)));$i++) {
        if ($current[$i]>$minimum[$i]) {
          $itis=true; // In this case, we have a winner
          break;
        }
        if ($current[$i]<$minimum[$i]) { // >
          $itis=false; // In this case, we have a loser
          break;
        }
      }
      if ($itis==2) { // This would only happen if all the common version
                      // components are identical. But there are may be
                      // differences:
                      // Example 1: comparing "4.0.1" with "4.0" would be
                      //            identical for now, but the condition
                      //            is satisfied;
                      // Example 2: comparing "4.0" with "4.0.1" - identical
                      //            for now, but the condition is NOT
                      //            satisfied
        if (sizeof($current)>=sizeof($minimum)) {
          $itis=true;
        } else {
          // Ok, only one more chance: if for example the user
          // specified "4.0.0" and phpversion returned "4.0".
          for ($i=sizeof($current)-1;$i<sizeof($minimum);$i++) { // >
            if ($minimum[$i]) {
              $itis=false;
            }
          }
          if ($itis==2) {
            $itis=true;
          }
        }
      }
      return($itis);
    }
  }

  if (!isphpver("4.0.2")) {
    // Word-wrap $string at $width chars
    // Already available in PHP 4.0.2 and later
    function wordwrap($string,$width=75)
    {
      // So, this is how we're going to go about wrapping the incoming
      // text:
      // We first break the text into natural lines:
      $tm_str=explode("\n",$string);
      for ($i=0;$i<sizeOf($tm_str);$i++) {
        // Then we move each line into a temporary string we'll start
        // chopping until we finish it.
        $temp_str=$tm_str[$i];
        $line_st="";
        while ($temp_str) {
          // Now let's see if the incoming string (or what's left of it)
          // isn't already shorter than the desired length.
          if (($line_len=strlen($temp_str))>$width) {
            // If it isn't, we'll just chop a piece of the ideal width
            // for starters.
            $wrapped_st=substr($temp_str,0,$width);

            // We'll start looking for a space in order to break the
            // line there.

            // (searching backwards!)
            if ($wrapping_len=strrpos($wrapped_st," ")) {
              // If there is one (it usually is) then we'll retrieve its
              // position and use it
              $line_len=$wrapping_len;
            } else {
              // Otherwise (i.e. a LOOONG word) then we'll just have to
              // look where it ends and use that.
              $line_len=strpos($temp_str," ");
              // But what if all we have is just a long word? No spaces, eh?
              if (!$line_len) {
                $line_len=strlen($temp_str);
              }
            }
          }
          // Now that we have the proper line ending we'll chop the
          // temporary string and add the chopped
          // part to the final (wrapped) string, along with a line
          // break.
          // Obviously, we'll do that in
          // reversed order than described here.
          $line_st.=substr($temp_str,0,$line_len)."\n";
          $temp_str=substr($temp_str,$line_len);

          // Making sure we don't leave spaces at the beginning of the
          // next line
          if (substr($temp_str,0,1)==" ") {
            $temp_str=substr($temp_str,1);
          }
        } // while
        // We need this for incoming empty lines:
        if (!strlen($line_st)) {
          $final_st.="\n";
        } else {
          $final_st.=$line_st;
        }
      } // for
      return($final_st);
    } // function

    // It just so happens that is_uploaded_file() has been implemented in 4.0.2
    // as well, so we'll use the same if() branch for this one as well
    function is_uploaded_file($filename)
    {
      if (!$tmp_file = get_cfg_var('upload_tmp_dir')) {
        $tmp_file = dirname(tempnam('', ''));
      }
      $tmp_file .= '/' . basename($filename);

      /* User might have trailing slash in php.ini... */
      return (ereg_replace('/+', '/', $tmp_file) == $filename);
    }
  } // if phpver

  // Get a value from the CRM database
  // usage: getvalue(table,value,id,[ifnull])
  // "value" should contain the field name
  // e.g. getvalue("people","fname",$CRM_user) will return
  // the first name of the current user.

  // If the value parameter is an array then an array with the same
  // size will be returned containing the values in the table for the
  // fields specified in the incoming array.

  function getvalue($table,$value,$id,$ifnull="n/a")
  {
    if ($id) {
      if (is_string($id)) {
        $theid="\"".$id."\"";
      } else {
        $theid=$id;
      }
      if (is_array($value)) {
        $values="";
        for ($i=0;$i<(sizeof($value)-1);$i++) { // >
          $values.=$value[$i].", ";
        }
        $values.=$value[$i];
      } else {
        $values=$value;
      }
      $sql="select $values from $table where id=$theid";
      $result=CRM_run_query($sql);
      if ($result) {
        if (OPT_num_rows($result)) {
          $myrow=OPT_fetch_row($result);
          if (!is_array($value)) {
            $myrow=$myrow[0];
          }
          return($myrow);
        }
      }
    } else {
      return($ifnull);
    }
  }

  // Irina Nikitina <irina@lanifex.com>
  function id_to_use($select_name,$txt_name,$table_name,$field_name,$id_field_name,$proj_id) {
    global $$select_name,$$txt_name;

    // Default assignment
    $id=$$select_name;
    $value=$$txt_name;

    // If there was input from text box
    if ($value!="") {
      // Check if not already there
      $sql="SELECT
              $id_field_name
            FROM
              $table_name
            WHERE
              $field_name='$value' AND
              (ProjectID=$proj_id OR ProjectID=0)";
      $result=CRM_run_query($sql);
      // If the value entered is already there return a corresponding id
      if (OPT_num_rows($result)!=0) {
        $myrow= OPT_fetch_row($result);
        $id=$myrow[0];
      } else {
        // If the value entered  is not there
        $proj_id=OPT_gpc_slashes($proj_id);
        $sql="INSERT INTO $table_name SET $field_name='$value', ProjectID=\"$proj_id\"";
        $result=CRM_run_query($sql);
        $sql="SELECT MAX($id_field_name) FROM $table_name WHERE $field_name='$value' AND ProjectID=\"$proj_id\"";
        $result=CRM_run_query($sql);
        $myrow= OPT_fetch_row($result);
        $id=$myrow[0];
      }
    }
    return $id;
  }

  //function to check if start date <= end date // >
  // Irina Nikitina <irina@lanifex.com>, November 2001
  function dates_ok($sd,$ed)
  {
    if (($sd==0) || ($ed==0)) {
      return true;
    } elseif ((strpos(" ".$sd,'-')) && (strpos(" ".$ed,'-'))){
      list($syear,$smonth,$sday)=explode('-',$sd);
      list($eyear,$emonth,$eday)=explode('-',$ed);
      $syear=intval($syear);
      $smonth=intval($smonth);
      $sday=intval($sday);
      $eyear=intval($eyear);
      $emonth=intval($emonth);
      $eday=intval($eday);
      if (($eyear>$syear) ||
          (($eyear==$syear) && ($emonth>$smonth)) ||
          (($eyear==$syear) && ($emonth==$smonth) && ($eday>=$sday))) {
        return(true);
      } else {
        return(false);
      }
    } else {
      if ($ed >= $sd) {
        return true;
      } else {
        return false;
      }
    }
  }  // end of dates_ok()

  if (!isphpver("4")) {
    // Only handles single-dimensional arrays and doesn't
    // have PHP 4's in_array's "strict" parameter.
    // But we don't need these, so it's ok.
    function in_array($needle, $haystack)
    {
      while (list($key,$val)=each($haystack)) {
        if ($val==$needle) {
          return(true);
        }
      }
      return(false);
    }
  }

  // Return the specified setting value from the database
  function get_setting($setting)
  {
    $value=false;
    if ($setting) {
      $sql="select value from settings where name=\"$setting\"";
      $result=crm_run_query($sql);
      if (OPT_num_rows($result)) {
        list($value)=OPT_fetch_row($result);
      }
    }
    return($value);
  }

  // Return the specified project setting value from the database
  function get_project_setting($proj, $setting)
  {
    $value=false;
    if ($setting) {
      $sql="select value from project_settings where project=$proj and name=\"$setting\"";
      $result=crm_run_query($sql);
      if (OPT_num_rows($result)) {
        list($value)=OPT_fetch_row($result);
      }
    }
    return($value);
  }

  // Return the specified user setting value from the database
  function get_user_setting($user, $proj, $type, $setting)
  {
    $value=false;
    if ($setting) {
      $sql="select value from user_settings where person=$user and project=$proj and type=\"$type\" and name=\"$setting\"";
      $result=crm_run_query($sql);
      if (OPT_num_rows($result)) {
        list($value)=OPT_fetch_row($result);
      }
    }
    return($value);
  }

  // Sets the given user setting value 
  function set_user_setting($user, $proj, $type, $name, $value)
  {
      $stype = OPT_gpc_slashes($type);
      $sname = OPT_gpc_slashes($name);
      $svalue = OPT_gpc_slashes($value);
      
      $sql = "delete from user_settings where person = $user and project = $proj and type = '$stype'
                 and name = '$sname'";
      crm_run_query($sql);
      
      $sql = "insert into user_settings (person, project, type, name, value) values ($user, $proj,
                 '$stype', '$sname', '$svalue')";
      crm_run_query($sql);
  }


  // Function returning all active people in the specified company
  // Company of zero means all people in all companies
  // $listType specifies the HTML type the list
  //   Valid values: MULTIPLE, SELECT
  // $varName is the name of the variable the result will
  // be passed in.
  // $idSelected is the id of the element selected (should be an array if MULTIPLE)
  // $size is used only in the MULTIPLE case, to specify the size of the window
  // $allow_none determines whether the first option should be nobody
  function get_people_of_comp($id, $listType=SELECT, $varName="people", $idSelected="-1", $size=5, $allow_none=0)
  {

    $out = "";
    $rows= "";
    $id=abs($id); // If the company asked for was 0, then get all they're allowed to see
    if ($id > 0)
    {
        $extra_sql = " and company = $id";
    }
    $result=crm_elements("people", "$extra_sql order by fname");

    while ($myrow=OPT_fetch_array($result)) {
      if ( (is_array($idSelected) && in_array($myrow["id"],$idSelected)) ||
           ($myrow[0] == $idSelected)) {
          $sel = "selected";
      }
      else
      {
          $sel="";
      }
      $rows .= "<option value=\"".$myrow["id"]."\" $sel>".$myrow["fname"].
            " ".$myrow["lname"]."</option>\n";
    }

    switch ($listType) {
      case MULTIPLE:
      $out .= '<select name="'.$varName.'" size="'.$size.'" multiple>'."\n";
      if ($allow_none)
      {
          $out .="<option value='0'>Nobody</option>";
      }
      $out .= $rows;
      $out .= '</select>'."\n";
      break;

      case SELECT:
      $out .= '<select name="'.$varName.'">'."\n";
      if ($allow_none)
      {
          $out .="<option value='0'>Nobody</option>";
      }
      $out .= $rows;
      $out .= '</select>'."\n";
      break;
    }
    return $out;
  }

  // This function saves an extra parameter for the specified
  // object. The function parameters are:
  // $table - the table in which the object to attach the parameter to is located;
  // $id - the id of that object in that table;
  // $class - the parameter class. This ensures that parameter names don't clash.
  //          Native OPT parameters will always be in a class starting with OPT_
  //          Plugin parameters are recommended to start with [name_of_plugin]_
  // $name - the parameter name
  // $value - the parameter value (treated as a string).
  // Please note that languages (the lang table) are the only one
  // in OPT which use two letters instead of an unsigned mediumint
  // as id - therefore you can only add extra parameters for those
  // by implementing a proprietary method to translate those two
  // letters into a word (two bytes).
  // Please note that the param_class is AFTER the param_name
  // when querying because of the way the key is made in the database.
  function write_Xtra($table, $id, $class, $name, $value)
  {
    $table=addslashes($table);
    $id=addslashes($id);
    $class=addslashes($class);
    $name=addslashes($name);
    $value=addslashes($value);
    OPT_run_query("delete from xtra_param where obj_type='$table' and
      obj_id='$id' and param_name='$name' and param_class='$class'");
    OPT_run_query("insert into xtra_param set obj_type='$table',
      obj_id='$id', param_class='$class', param_name='$name',
      param_value='$value'");
  }

  function read_Xtra($table, $id, $class, $name)
  {
    $table=addslashes($table);
    $id=addslashes($id);
    $class=addslashes($class);
    $name=addslashes($name);
    $result=OPT_run_query("select param_value from xtra_param
      where obj_type='$table' and obj_id='$id' and
      param_name='$name' and param_class='$class'");
    if (list($value)=OPT_fetch_row($result)) {
      return($value);
    } else {
      return(false);
    }
  }

  // This function returns an array in human-readable format
  // Similar to var_dump(), but this one returns the string instead
  // of actually dumping it.
  function OPT_dump_array($array,$indent="")
  {
    while(list($key,$val)=each($array)) {
      $result.=$indent."[$key] = ";
      if (is_array($val)) {
        $result.="{\n".OPT_dump_array($val,$indent."\t")."$indent}\n";
      } else {
        $result.=$val."\n";
      }
    }
    return($result);
  }

  // < --- end of really generic functions --- >

  // =====================================================================
  //
  //                            Mail functions
  //
  // =====================================================================

  // This is the function used to send mail throughout OPT. It has
  // precisely the same syntax as mail() - the nice thing is that
  // it sends the correct character encoding provided in the globals.
  function OPT_mail($to, $subject, $message, $ah="", $ap="")
  {
    $tah=$ah;
    if (!(($tah) && (substr($tah,-1,1)=="\n"))) {
      $tah.="\n";
    }
    global $_OPT;
    $tah.="Content-Type: text/plain; charset=".$_OPT["local"]["encoding"];
    if ((isphpver("4.0.5")) && ($ap)) {
      return(mail($to, $subject, $message, $tah, $ap));
    } else {
      // additional parameters only in PHP>4.0.5
      // plus some problems with using an empty $ap in some PHP versions
      // (reported for 4.1.2)
      return(mail($to, $subject, $message, $tah));
    }
  }

  // Notification function - sends e-mail notifications
  // Incoming parameters:
  // - e-mail
  // - subject
  // - message body [NOT wrapped!]
  // - e-mail instructions body
  // - "from" e-mail
  // Returns 0 on success, 1 on failure and 2 on disabled notifications
  // Please note this function MUST NOT be used with administrative notifications
  // because it can be inhibited by the notify setting
  function OPT_notify($dest, $subject, $body, $ibody, $from, $inhibit_confirmations=0, $ignore_notify=0)
  {
    global $CRM_user;

    $udata[]="email";
    $udata[]="sal";
    $udata[]="fname";
    $udata[]="lname";
    $udata=getvalue("people",$udata,$dest);
    $email="$udata[2] $udata[3] <$udata[0]>";
    $name=htmlspecialchars("$udata[1] $udata[2] $udata[3]");
    if ((!get_setting("notify")) && (!$ignore_notify)) {
      $result=2;
    } elseif ((getvalue("people","status",$dest) & 8) && (!$ignore_notify)) {
      $result=4;
    } elseif (($dest == $CRM_user) && (!$inhibit_confirmations)) {
      $result=8;  // don't send notification to the current user, unless we're inhibiting confirmations
                  // in which case this could a notify from email.php which the current user should see
    } else {
      if (!get_setting("mail_interface")) {
        $ibody="";
      }
      if ($ibody) {
        $body.="\n\n\n\n\n\n".
        "__________________________________________________________________________ ".
        $ibody;
      }
      global $CRM_demo;
      if ($CRM_demo) {
        $result=0;
      } elseif (OPT_mail($email,$subject,wordwrap($body),"From: $from")) {
        $result=0;
      } else {
        $result=1;
      }
    }
    if ((!$inhibit_confirmations) && (crm_perm("people",$dest))) {
      switch($result)
      {
        case 0: // Success
          OPT_confirm("Sent notification to $name");
          break;
        case 1: // Failure
          OPT_error("Unable to send notification to $name [mail error]");
          break;
        case 2: // Disabled [system-wide]
          OPT_confirm("Didn't send notification to $name due to system settings");
          break;
        case 4: // Disabled [for this user]
          OPT_confirm("Didn't send notification to $name due to user settings");
          break;
        case 8: // Don't send out notifications to the current user (they know what they just did)
          break;
        default: // WTF?
          OPT_error("Tried to send notification to $name but the mail function returned an unknown message. This is an unknown bug!");
          break;
      }
    }
    return($result);
  }

  // < --- end of mail functions --- >

  // =====================================================================
  //
  //                       Database-related functions
  //
  // =====================================================================

  // This function tries to send the database error message to the OPT administrator
  function CRM_query_error($err_msg)
  {
    global $CRM_email, $CRM_system_email, $CRM_user,
          $REQUEST_URI, $SERVER_NAME, $HTTP_REFERER,
          $HTTP_GET_VARS, $HTTP_POST_VARS, $HTTP_USER_AGENT,
          $CRM_ver, $CRM_subver, $OPT_db_lock_bypass;

    ######################################################
    #                                                    #
    # ------------------ CAUTION! ---------------------- #
    #                                                    #
    # We force unlocking the database because we need to #
    # check  some  global  settings - make sure we never #
    # try to edit data in this function!                 #
    #                                                    #
    ######################################################
    $orig_bypass=$OPT_db_lock_bypass;
    $OPT_db_lock_bypass=true;

    if ($HTTP_REFERER) {
      $ref=$HTTP_REFERER;
    } else {
      $ref="an unknown location";
    }
    $user[]="fname";
    $user[]="lname";
    $user=getvalue("people",$user,$CRM_user,"an unknown or guest user");
    $user="$user[0] $user[1]";
    $html_dump="<hr>\n".nl2br($err_msg)."<br>\n";
    $html_dump.="<p>This error occured while $user was accessing $REQUEST_URI\n";
    $html_dump.="<p>The user came here from $ref\n";
    $html_dump.="<p>Please make sure you include the lines above when submitting a bug report!";
    $html_dump.="<hr>\n";

    if ((strlen(strstr($REQUEST_URI,"setup/")) == 0) && (!get_setting("no_SQL_errors")))
    {
        echo($html_dump);
        $error_dumped=true;
    } 
    else 
    {
        $error_dumped=false;
    }

    while (list($key,$val)=each($HTTP_GET_VARS)) {
      if (is_array($val)) {
        $GetVars.="\t$key = {\n".OPT_dump_array($val,"\t\t")."\t}\n";
      } else {
        $GetVars.="\t$key = $val\n";
      }
    }
    while (list($key,$val)=each($HTTP_POST_VARS)) {
      if (is_array($val)) {
        $PostVars.="\t$key = {\n".OPT_dump_array($val,"\t\t")."\t}\n";
      } else {
        $PostVars.="\t$key = $val\n";
      }
    }
    $mail_dump="The OPT system at $SERVER_NAME issued a database error:\n".
      "--------------------------\n".
      "$err_msg\n".
      "--------------------------\n".
      "This error occured while $user was accessing $REQUEST_URI\n".
      "The user came here from $ref\n".
      "OPT $CRM_ver $CRM_subver\n".
      ($GetVars?"GET variables:\n$GetVars\n":"").
      ($PostVars?"POST variables:\n$PostVars\n":"").
      "Register globals: ".(ini_get("register_globals")?"On":"Off")."\n".
      "User Agent: $HTTP_USER_AGENT";
    $sent=OPT_mail($CRM_email,"OPT Error!",
      wordwrap($mail_dump),
      "From: OPT system <$CRM_system_email>");
    if ($sent) {
      OPT_confirm("The message has been successfully sent to the OPT administrator.");
    } else {
      OPT_error("Unable to send error message to the OPT administrator - please try to reach him/her at ".
        "<a href='mailto:$CRM_email'>$CRM_email</a> and report this message:");
      echo(nl2br($mail_dump));
    }
    $OPT_db_lock_bypass=$orig_bypass;
  }

  // This function removes slashes to from string passed to it only if
  // PHP variable magic_quotes_gpc is set.
  // Obviously, the string should come from a GET, POST or Cookie code
  // for this action to make sense.
  function OPT_gpc_unslash($slashed)
  {
    global $OPT_force_slashing;
    if ((!$OPT_force_slashing) && (isphpver("4"))) {
      if (ini_get("magic_quotes_gpc")) {
        return(stripslashes($slashed));
      } else {
        return($slashed);
      }
    } else {
      return($slashed);
    }
  }

  // This function adds slashes to the string passed to it only if
  // PHP variable magic_quotes_gpc is set.
  // Obviously, the string should come from a GET, POST or Cookie
  // for this action to make sense.
  function OPT_gpc_slashes($unslashed)
  {
    global $OPT_force_slashing;
    if ((!$OPT_force_slashing) && (isphpver("4"))) {
      if (ini_get("magic_quotes_gpc")) {
        return($unslashed);
      } else {
        return(addslashes($unslashed));
      }
    } else {
      return(addslashes($unslashed));
    }
  }

  // This function adds slashes to the string passed to it only if
  // PHP variable magic_quotes_runtime is set.
  // Obviously, the string should come from a database or from mail
  // for this action to make sense.
  function OPT_runtime_slashes($unslashed)
  {
    global $OPT_force_slashing;
    if ((!$OPT_force_slashing) && (isphpver("4"))) {
      if (ini_get("magic_quotes_runtime")) {
        return($unslashed);
      } else {
        return(addslashes($unslashed));
      }
    } else {
      return(addslashes($unslashed));
    }
  }

  // This function declares all necessary variables for your query
  // This function was inserted here by Irina Nikitina <irina@lanifex.com> but
  // I can't tell if she wrote it
  function make_vars($result)
  {
    $fields = OPT_num_fields($result);
    if ($myrow = OPT_fetch_array($result)) {
      for ($i=0; $i < $fields; ++$i){ // >
        $names = OPT_fieldname($result,$i);
        global $$names;
        $$names = $myrow[$names];
      }
      return true;
    } else {
      return false;
    }
  }

  // < --- end of database-related functions --- >

  // =====================================================================
  //
  //                            IP functions
  //
  // =====================================================================

  // Checks if the IP is a valid one - i.e. if it contains three periods and
  // only digits. An IP such as "192.291.332.192, 193" (reported by a user)
  // will be returned as "192.291.332.192".
  function valid_IP($ip)
  {
    if (!preg_match("/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/",$ip,$valid)) {
      return(false);
    } else {
      return($valid[0]);
    }
  }

  // Returns the correct remote IP
  function OPT_remote_IP()
  {
    global $HTTP_X_FORWARDED_FOR, $HTTP_VIA, $REMOTE_ADDR;

    if(($ip=$HTTP_X_FORWARDED_FOR) && (($ip=valid_IP($ip)) || (($ip=gethostbyname($HTTP_X_FORWARDED_FOR)) && ($ip=valid_IP($ip)))));
    elseif(($ip=$HTTP_VIA) && (($ip=valid_IP($ip)) || (($ip=gethostbyname($HTTP_VIA)) && ($ip=valid_IP($ip)))));
    elseif(($ip=$REMOTE_ADDR) && (($ip=valid_IP($ip)) || (($ip=gethostbyname($REMOTE_ADDR)) && ($ip=valid_IP($ip)))));
    else {
      // Ugly way of saying we don't know the IP
      // The nice thing about it is that it still can be treated
      // as any other IP (i.e. exploded by periods and maked)
      $ip = "0.0.0.0";
    }
    return($ip);
  }

  // < --- end of IP functions --- >

  // =====================================================================
  //
  //                    Internationalization functions
  //
  // =====================================================================

  // Calling this function will set all the locales properly for the
  // user specified. If the user is ommitted, the locales are set for
  // the current user.

  // The default locales are set using this function by auto_open() in auth.php
  // because that's where we find out who the current user is.

  // You may use this if you interact with some other user (such as sending an e-mail).
  // Just make sure you call it once again with no parameters after finishing with
  // that user.
  function OPT_locale($userid=0)
  {
    if (!function_exists(bindtextdomain)) {
      return(false);
    }
    if (!$userid) {
      global $CRM_user;
      $userid=$CRM_user;
    }
    global $_OPT;
    $lang=getvalue("people","preflang",$userid);
    if (!$lang) {
      $lang="en";
    }
    $encoding=getvalue("lang","encoding",$lang);
    if (!$encoding) {
      $encoding="ISO-8859-1";
    }
    $_OPT["local"]["lang"]=$lang;
    $_OPT["local"]["encoding"]=$encoding;
    putenv("LANG=$lang");
    setlocale(LC_ALL, $language);
    if (!bindtextdomain("OPT_local","OPT_lang")) {
      return(false);
    }
    textdomain("OPT_local");
  }

  // Defining it conditionally instead of checking at every call
  if (function_exists(_)) {
    function __($msg) {
      return(_($msg));
    }
  } else {
    function __($msg) {
      return($msg);
    }
  }

  // This function returns the timezone in minutes when fed an OPT-like timezone
  function OPT_tz_to_sec($tz)
  {
    $tz_s=substr($tz,0,1);
    $tz_h=substr($tz,1,2);
    $tz_m=substr($tz,3,2);
    if (($tz_s!='+') && ($tz_s!='-')) {
      $tz_s='+';
      $tz_h=substr($tz,0,2);
      $tz_m=substr($tz,2,2);
    }
    $tz_minutes=($tz_s."1")*($tz_h*60+$tz_m);
    return($tz_minutes*60);
  }

  // Function which returns a PHP timestamp of a date in the specified
  // time zone. If the date is not specified, the current system date
  // is used. If you do pass a date, make sure it's UTC.
  // The time zone must be in numeric format - "+hhmm" or "-hhmm".
  // "hhmm" is also accepted, and assumed to be identical to "+hhmm".
  function OPT_local_date($tz, $date=false)
  {
    if (!$date) {
      $date=time()-date('Z');
    }
    return($date+OPT_tz_to_sec($tz));
  }

  // < --- end of internationalization functions --- >

  // =====================================================================
  //
  //                         Table headers functions
  //
  // =====================================================================

  function OPT_header_cell($cat,$order,$text,$link_extra,$colspan=0,$rowspan=0)
  {
    global $PHP_SELF, $OPT_col_dk, $OPT_icons, $CRM_URL, $OPT_col_attention, $pf;
    if ($colspan) {
      $span=" colspan='$colspan'";
    }
    if ($rowspan) {
      $span.=" rowspan='$rowspan'";
    }
    $cell_start="  <td class='regular' align='center' bgcolor='$OPT_col_dk'$span>";
    $cell_end="</td>\n";
    $order_plus=$order_orig=$order;
    if (!$cat) {
      // No category - no link
      return("$cell_start<b>$text</b>$cell_end");
    } elseif(@in_array($cat,$order)) {
      // If we already order by this category then we'll swap the direction
      // (see the following elseif as well)
      $i=0;
      while ($order[$i]!=$cat) $i++;
      $order[$i]="rev_$cat";
      $icon=$OPT_icons["header"]["arrow_down"];
    } elseif(@in_array("rev_$cat",$order)) {
      $i=0;
      while ($order[$i]!="rev_$cat") $i++;
      $order[$i]=$cat;
      $icon=$OPT_icons["header"]["arrow_up"];
    } else {
      // Finally, if there is a category specified and we're not currently
      // using it for sorting then we produce the plus icon link and, if
      // clicking on the text link, we replace the current order, whatever it was
      $order_plus[]=$cat;
      $order=array($cat);
    }
    $result=$cell_start;
    $result.="<table border='0' cellspacing='0' cellpadding='0' width='100%'>";
    $result.="<tr>";

    // The figure
    if (isset($i) && (sizeOf($order)>1)) {
      $result.="<td align='left'>";
      if ($pf) {
        $result.="[".($i+1)."]";
      } else {
        $result.="&nbsp;<img src='$CRM_URL/images/".$OPT_icons["header"]["figures"][$i+1]."'>";
      }
      $result.="</td>";
    }

    // The main entry
    $result.="<td align='center' width='100%'>";
    $result.="<A href='$PHP_SELF?order=".OPT_parse_order_array($order)."&$link_extra' class='regular'>".
      "<b>$text</b></A>";
    $result.="</td>";

    if ($icon) {
      // The order icon (ascending/descending)
      $j=0;
      $alt_order=array();
      while($j<$i) {
        $alt_order[]=$order[$j];
        $j++;
      }
      $j++;
      while($j<sizeOf($order)) {
        $alt_order[]=$order[$j];
        $j++;
      }
      $result.="<td align='right'>";
      if (!$pf) {
        $result.="<A HREF='$PHP_SELF?order=".OPT_parse_order_array($alt_order)."&$link_extra' class='regular'$span>";
        $result.="<IMG SRC='$CRM_URL/images/$icon' border='0'>&nbsp;";
        $result.="</A>";
      } else {
        if ($icon==$OPT_icons["header"]["arrow_down"]) {
          $result.="(&gt;)";
        } else {
          $result.="(&lt;)";
        }
      }
      $result.="</td>";
    } elseif ((sizeOf($order_orig)) && (!$pf)) {
      // Or the plus sign if we're not ordering by this category
      $result.="<td align='right'>";
      $result.="<A HREF='$PHP_SELF?order=".OPT_parse_order_array($order_plus)."&$link_extra' class='regular'$span>";
      $result.="<IMG SRC='$CRM_URL/images/".$OPT_icons["header"]["plus"]."' border='0'>&nbsp;";
      $result.="</A>";
      $result.="</td>";
    }

    // Ending the table
    $result.="</tr>";
    $result.="</table>";
    $result.=$cell_end;
    return($result);
  }

  // Used to determine whether a select option is selected 
  function isSel($variable, $target)
  {
      if ($variable == $target)
          echo ('selected');
  }
  
  function OPT_parse_order_string($str)
  {
    if (!$str) {
      return(array());
    }
    return(explode("|",$str));
  }

  function OPT_parse_order_array($arr) {
    return(implode("|",$arr));
  }

  function OPT_build_order_query($order)
  {
    if ((!is_array($order)) || (!sizeOf($order))) {
      return("");
    }
    while(list(,$cat)=each($order)) {
      if (substr($cat,0,4)=="rev_") {
        $cat_query=substr($cat,4)." desc";
      } else {
        $cat_query=$cat." asc";
      }
      $query.=", $cat_query";
    }
    return("order by ".substr($query,2));
  }

  // < --- end of table headers functions --- >

  // =====================================================================
  //
  //                            Autorun code
  //
  // =====================================================================

  // Now, this is an ugly thing to do, but we'll have to do it for this version of OPT.
  // We're going to emulate register_global_vars if it isn't set... We only do it if
  // the global $OPT_no_globals is NOT set - this way we can incrementally upgrade OPT
  // to work with register_globals off.
  if ((!ini_get("register_globals")) && (!$OPT_no_globals)) {
    extract($HTTP_GET_VARS);
    extract($HTTP_POST_VARS);
    extract($HTTP_SERVER_VARS);
  }

  // < --- end of autorun code --- >

// include some more common methods
if ($CRM_inc)
{
    include("$CRM_inc/common.php");
}
else
{
    include("$inc/common.php");
}


?>
