<?
#
# User rights library
#
include("$CRM_inc/elem_perm.inc");  // individual methods for determining elemement specific permissions
#
# All functions by both
#   Bogdan Stancescu <bogdan@lanifex.com>, November 2001
#   Andrei Tinca <andrei@lanifex.com>, November 2001
# unless otherwise specified.
# Revision 1.0
// Returns 1 if user belongs is admin. This function is hardly ever used
// which is the wrong way to code. Hummm...
// Andrei Tinca <andrei@lanifex.com>, November 2001
function user_comp()
{
    global $CRM_comp;
    if ($CRM_comp==1)
    {
        return(1);
    }
    else
    {
        return(0);
    }
}

// Indicates if user has superuser or 'root' privileges
function is_admin()
{
    global $CRM_comp, $secuLevel, $CRM_user;
    return (!(getvalue("people","status",$CRM_user) & ADMIN));
}

// Returns true if the current user is the project manager for the given project
function is_project_manager($proj)
{
    global $CRM_user;
    $sql="select count(*) from project_managers where project=$proj and manager=$CRM_user";
    $result=CRM_run_query($sql);
    $row=OPT_fetch_row($result);
    if ($row[0] > 0)
    {
        return(1);  // This is the project manager
    }
    else
    {
        return(0);  // This is not the project manager
    }
}

// Tests what the current user's permissions are for message $id
// Returns an integer with the following permission bits flagged:
// (square brackets for the test you have to perform in order to see
// if that bit is flagged)
// 0 [& 1] - read
// 1 [& 2] - post reply
// 2 [& 4] - delete
// 3 [& 8] - see user who posted
// 4 [&16] - reattach
// 5 [&32] - subscribe
function forum_perm($id)
{
    if (system_perm("forums")==65535)
    {
        return(65535);
    }

    // We have to first find out what this message is attached to
    // and find the permissions for that
    $att_prop[]="att_type";
    $att_prop[]="att_id";
    $att_prop=getvalue("forums",$att_prop,$id);
    if ($att_prop[0]!="system")
    {
        $perm=crm_perm($att_prop[0],$att_prop[1]);
        if ($perm)
        {
            // We may get other permissions than 0 or 1, but we only
            // need the read permission right now
            $perm=1;
            $perm+=32; // We also allow subscribing
        }
    }
    else
    {
        $perm=1;
    }

    // Ok, can we post replies?
    global $CRM_guest;
    if (!$CRM_guest)
    {
        $perm+=2;
    }

    // Can we delete or reattach?
    global $CRM_user;
    $src=getvalue("forums","src",$id);
    if ($src==$CRM_user)
    {
        $perm+=4;
        $perm+=16;
    }

    // Can we see who posted?
    if (crm_perm("people",$src))
    {
        $perm+=8;
    }

    return($perm);
}

// Find out which elements of a specific table
// are accessible to the current user
// Added extra_sql to tack on the end of the sql statement for order and extra conditions
// If you don't pass it to the function nothing happens, it's optional
function crm_elements($table, $extra_sql="")
{
    global $CRM_user, $CRM_comp, $proj, $CRM_guest;

    // Administrative users
    if (is_admin() || $CRM_guest) 
    {
        return elems_for_admin($table, $extra_sql);
    } 
    else // regular users
    {
        // this is trickier -- we've got to consider different cases:
        if ($table=="people")
        {
            return elems_for_people($table, $extra_sql);
        } 
        elseif ($table=="companies") 
        {
            return elems_for_companies($table, $extra_sql);
        } 
        elseif ($table=="projects") 
        {
            return elems_for_projects($table, $extra_sql);
        } 
        elseif ($table=="documents")
        {
            return elems_for_documents($table, $extra_sql);
        }
        elseif ($table=="requests")
        {
            return elems_for_requests($table, $extra_sql);
        } 
        elseif ($table=="email") 
        {
            return elems_for_email($table, $extra_sql);
        } 
        elseif ($table=="people") {
            return elems_for_people($table, $extra_sql);
        }
        elseif ($table=="task") {
            return elems_for_tasks($table, $extra_sql);
        }
    }
    return $result;
}

// Returns system-wide *admin* permissions based on the table.
// The result is an integer with the following bits set:
// 0 [& 1] - read permission (can read everything in this table)
// 1 [& 2] - write permissions (can write anywhere in this table)
// 2 [& 4] - edit permissions (can edit any element in this table)
// 3 [& 8] - delete permissions (can delete anything in this table)
// Tables that can be specified are : people, login, forums, ...
function system_perm($table)
{
    global $CRM_admin, $secuLevel;

    if ($table == "login") {
        if (is_admin()) {
            return 65535;
        } else {
            return 0;
        }
    }

    if ($CRM_admin["system"]) {
        return(65535);
    }
    if (($table=="forums") && ($CRM_admin["forums"])) {
        return(65535);
    }
}

// Find out if a specific element in a specific table
// is accessible to the current user
// if returns 0, not allowed
// 1 : all rights
// 2 : edit but not delete
// 3 : view only
// if $action is specified, then returns the result as
// a bool for the specified action (VIEW, EDIT, ALL).
function crm_perm($table,$table_id, $action=-1)
{
    global $CRM_user,$CRM_comp, $CRM_guest;

    // By default you're not allowed to see element table_id
    $allow=0;
    if ($CRM_guest) {
        $allow=3; // Only read access
    } elseif (is_admin()) {
        // This means s/he's got root - and admins can see everything
        $sql="select id from $table where id = $table_id";
        $result=CRM_run_query($sql);
        if ($myrow=OPT_num_rows($result))
        {
            $allow=1;
        }
    } else {
        // this means they're a regular user, so we've got to consider different cases:
        if ($table=="people") 
        {
            $allow=elem_perm_for_people($table_id);
        } 
        elseif ($table=="companies") 
        {
            $allow=elem_perm_for_companies($table_id);
        } 
        elseif ($table=="projects")
        {
            $allow=elem_perm_for_projects($table_id);
        } 
        elseif ($table=="documents") {
            $allow=elem_perm_for_documents($table_id);
        }
        elseif ($table=="requests") 
        {
            $allow=elem_perm_for_requests($table_id);
        } 
        elseif ($table=="email") 
        {
            $allow=elem_perm_for_email($table_id);
        } 
        elseif ($table=="task") {
            $allow=elem_perm_for_task($table_id);
        } 
        elseif ($table=="meeting") {
            $allow=elem_perm_for_meeting($table_id);
        }
        elseif ($table=="events") {
            $allow=elem_perm_for_events($table_id);
        }
        elseif ($table=="absences") {
            $allow=elem_perm_for_absences($table_id);
        }
        elseif ($table=="code_repositories") {
            $allow=elem_perm_for_code_repositories($table_id);
        }  
    }
    
    
    switch($action) {
    case ALL:
        return ($allow == 1);
        break;
    case EDIT:
        return (($allow == 2) || ($allow == 1));
        break;
    case VIEW:
        return (($allow == 3) || ($allow == 2) || ($allow == 1));
        break;
    default:
        return $allow;
    }
}


// Determine whether this user is a "participant" in this project in either
// the role of developer or customer
// Return True or False depending on whether this user was added to the project
function crm_project_participant($userid, $proj, $role=DEVELOPER)
{
    $company = getvalue("people", "company", $userid);

    if ($role == DEVELOPER)
    {
        $role_sql="Developer";
    }
    else
    {
        $role_sql="Customer";
    }

    $sql="select count(*) num from company2project where company=$company and project=$proj and role='$role_sql'";
    $result=CRM_run_query($sql);
    $rec = OPT_fetch_array($result);
    if ($rec && ($rec["num"] == 1))
    {
        return 1;  // User is in a group with access to project
    }

    $sql="select count(*) num from people_proj where person=$userid and project=$proj and role='$role_sql'";
    $result=CRM_run_query($sql);
    $rec = OPT_fetch_array($result);
    if ($rec && ($rec["num"] == 1))
    {
        return 1;  // User is an indiviudal with access to the project
    }
    else
    {
        return is_project_manager($proj);
    }
}

// Indicates whether this person is a PM on any project at all
function person_is_project_manager($person)
{
    // Determine if this user is a PM on any project
    $sql="select count(*) from project_managers, projects where project_managers.project = projects.id 
          and manager = $person and projects.active = 1";
    $result=CRM_run_query($sql);
    $row=OPT_fetch_row($result);
    if ($row[0] > 0)
    {
        return 1;  // They're a PM on at least one project
    }
    return 0;
}

// Indicates whether this person has the role of developer (team member) on any project
function person_is_developer($person)
{
    // Determine if this user is a team member on any project based on their group
    $person_comp=getvalue("people", "company", $person);
    $sql="select count(*) from company2project where company = $person_comp and 
                 company2project.role = 'Developer'";
    $result=CRM_run_query($sql);
    $row=OPT_fetch_row($result);
    if ($row[0] > 0)
    {
        return 1;  // Person is on a worker on at least one project
    }
    
    // Not a team member by group, check if one as indvidual
    $sql="select count(*) from people_proj
          where person = $person and people_proj.role = 'Developer'";
    $result=CRM_run_query($sql);
    $row=OPT_fetch_row($result);
    if ($row[0] > 0)
    {
        return 1;  // Person is on a worker on at least one project
    }

    return 0;
}

// Indicates whether this person has the role of payroll contact on any company (group)
function person_is_payroll_contact($person)
{
    // Determine if this user is a team member on any project
    $sql="select id from companies where payroll_contact = $person";
    $result=CRM_run_query($sql);
    $row=OPT_fetch_row($result);
    if ($row[0] > 0)
    {
        return 1;  // Person is payroll contact for at least one company
    }
    return 0;
}

// Indicates whether this person has the role of group contact on any company (group)
function person_is_group_contact($person)
{
    // Determine if this user is a team member on any project
    $sql="select id from companies where contact = $person";
    $result=CRM_run_query($sql);
    $row=OPT_fetch_row($result);
    if ($row[0] > 0)
    {
        return 1;  // Person is contact for at least one company
    }
    return 0;
}
?>