<?

// Setup Gantt graph
$graph = new GanttGraph(700,0,'auto');
$graph->SetShadow();
$graph->SetBox();


// If we still don't have a start and end date, then let's use some defaults
if (!$start_date || !$end_date)
{  
    $start_date = date("Y-m-d", mktime(0,0,0, date(m)-1, date(d),date(Y))); // One month ago 
    $end_date = date("Y-m-d", mktime(0,0,0, date(m)+1, date(d),date(Y)));   // to One month from now
}

// Only show part of the Gantt
$graph->SetDateRange($start_date,$end_date);

// Determine which header for dates to show
if ((strtotime($end_date) - strtotime($start_date)) > 60*60*24*365)  
{   // More than two years difference
    $graph->ShowHeaders(GANTT_HYEAR);
}
else if ((strtotime($end_date) - strtotime($start_date)) > 60*60*24*365*(5/12))  
{   // More than five months
    $graph->ShowHeaders(GANTT_HYEAR | GANTT_HMONTH);
}
else if ((strtotime($end_date) - strtotime($start_date)) > 60*60*24*365*(2/12))  
{   // More than two months
    $graph->ShowHeaders(GANTT_HYEAR | GANTT_HMONTH | GANTT_HWEEK);
}
else
{   // Less than two months
    $graph->ShowHeaders(GANTT_HYEAR | GANTT_HMONTH | GANTT_HDAY | GANTT_HWEEK);
}

$graph->scale->week->SetStyle(WEEKSTYLE_FIRSTDAY);
$graph->scale->week->SetFont(FF_FONT1);

$bar_index = 0;  // incremented for each bar on chart

// Get all milestones for all projects this person is involved with as a developer
// that fall within this time range
$proj_ids = get_projects_with_developer_access($person);

if ($proj_ids)
{
    $sql="select due_date, name from project_milestones where project in (".implode(",", $proj_ids).") 
     and due_date >= '".OPT_gpc_slashes($start_date)."' and due_date <= '".OPT_gpc_slashes($end_date)."'";
    $result=CRM_run_query($sql);
    while ($row = OPT_fetch_row($result))
    {
        $ms = new MileStone($bar_index++,"",$row[0],date("M d",strtotime($row[0])));
        $ms->title->SetFont(FF_FONT1,FS_BOLD);
        $graph->Add($ms);  // Add the milestone
        
        $vline = new GanttVLine($row[0], $row[1], "darkred"); // add it as a line as well to really stand out
        $vline->title->SetFont(FF_FONT1,FS_BOLD,10);
        $vline->SetDayOffset(0.5);    // Center the line in the day
        $graph->Add($vline);
    }
}

// Now get all tasks that fall within this time period and save them
$sql="select distinct(task.id) as id
        from task LEFT JOIN person_task ON task.id = person_task.taskID
        where personID = $person and 
           active='1'  and  ((StartDate > 0) and (StartDate < UNIX_TIMESTAMP('$end_date'))
           OR (StartDatePlanned > 0) and (StartDatePlanned < UNIX_TIMESTAMP('$end_date'))) and
           (EndDate > UNIX_TIMESTAMP('$start_date') OR EndDatePlanned > UNIX_TIMESTAMP('$start_date'))";
$result=crm_run_query($sql);
while ($row=OPT_fetch_array($result))
{
    $task_ids[] = $row["id"];
}

// Now get all requests that are associated with these tasks
if ($task_ids)
{
    $sql = "select distinct(requests.id) as rid, title, date, deadline, priority, aproj 
            from requests, request2task where requests.id = request2task.request and
            task in (".implode(",", $task_ids).") order by priority, deadline";
    $result = OPT_run_query($sql);
    $last_priority = -1;
    while($row=OPT_fetch_array($result))
    {
        // Need to get settings from the project for this request
        $proj=$row["aproj"];
        $statii=get_project_setting($proj, "request_status_options");
        $extra_statuses=explode("|", $statii); 
        $priorities = get_project_setting($proj, "request_priority");
        if (!$priorities) {
            $priorities=array();
        }
        $priorities = explode("|",$priorities);

        // Get an array of start, end, progress, caption from its tasks
        $details = calcRequestDetails($row["rid"], $row["date"], $row["deadline"]);
        
        // Only bother adding it to the graph if it will be displayed
        if ((strtotime($details[0]) > 0) && (strtotime($details[1]) > 0))
        {
            if ($last_priority != intVal($row["priority"]))
            {
                // Use milestones without date to indicate a request priority
                $last_priority = intVal($row["priority"]);
                $ms = new MileStone($bar_index++,$priorities[$row["priority"]],0,"");
                $ms->title->SetFont(FF_FONT1,FS_BOLD);
                $ms->title->SetColor("black");
                $graph->Add($ms);  // Add the milestone
            }
            
            $bar = new GanttBar($bar_index++,"Request #".$row["rid"],$details[0],$details[1],$details[3]);
            
            // Set the progress amount on the bar
            $bar->SetPattern(BAND_RDIAG,"yellow");
            $bar->SetFillColor("red");
            $bar->progress->Set($details[2]);
            $bar->progress->SetPattern(GANTT_SOLID,"darkgreen");
            
            // Now add a Client side image map to link directly to task with hovering title
            $bar ->SetCSIMTarget ("../../rt/req_view/index.php?proj=$proj&req=".$row["rid"]);
            $bar->SetCSIMAlt(str_replace("\"", "'", $row["title"]));
            $bar->title-> SetCSIMTarget("../../rt/req_view/index.php?proj=$proj&req=".$row["rid"]);
            $bar->title-> SetCSIMAlt("Click to view details");

            // Since these are requests wrapping a set of tasks, add contrainst indicators
            $bar->leftMark->SetColor("black");
            $bar->leftMark->SetFillColor("red");
            $bar->leftMark->SetWidth(5);
            $bar->leftMark->SetType(MARK_LEFTTRIANGLE);
            $bar->leftMark->Show();            
            $bar->rightMark->SetColor("black");
            $bar->rightMark->SetFillColor("red");
            $bar->rightMark->SetWidth(5);
            $bar->rightMark->SetType(MARK_RIGHTTRIANGLE);
            $bar->rightMark->Show();

            $graph->Add($bar);
            
            // Now show all tasks for this request below it (inside constraints)
            // Get all the tasks associated with this request
            $sql="select distinct(task.id) as id, TaskName, StartDate, StartDatePlanned, 
            EndDate, EndDatePlanned, PercentComplete from request2task,       
            task LEFT JOIN person_task ON task.id = person_task.taskID
            where request2task.task = task.id and request = ".$row["rid"]."
            and (StartDate > 0 OR StartDatePlanned > 0) and 
            (EndDate > 0 OR EndDatePlanned > 0) and person_task.personID = $person
            order by StartDate, StartDatePlanned";
            $task_result = CRM_run_query($sql);
            while ($task_row = OPT_fetch_array($task_result))
            {
                // Determine the start date (real overrides planned)
                if ($task_row["StartDate"])
                {
                    $task_start = date("Y-m-d", intVal($task_row["StartDate"]));
                }
                else
                {
                    $task_start = date("Y-m-d", intVal($task_row["StartDatePlanned"]));
                }
                
                // Determine the end date (real overrides planned)
                if ($task_row["EndDate"])
                {
                    $task_end = date("Y-m-d", intVal($task_row["EndDate"]));
                }
                else
                {
                    $task_end = date("Y-m-d", intVal($task_row["EndDatePlanned"]));
                } 
                
                // Add a caption indicating the percentage of progress if it's in progress
                if ($task_row["PercentComplete"] && ($task_row["PercentComplete"] < 100))
                {
                    $caption = " [" . $task_row["PercentComplete"] ."%]";
                }
                else
                {   // Don't show 100% complete as color makes it obvious
                    $caption = "";
                }    
                
                $bar = new GanttBar($bar_index++,"  Task #".$task_row["id"],$task_start,$task_end, $caption);
                
                // Set the progress amount on the bar
                $bar->SetPattern(BAND_RDIAG,"red");
                $bar->SetFillColor("yellow");
                $bar->progress->Set($task_row["PercentComplete"]/100);
                $bar->progress->SetPattern(GANTT_SOLID,"darkblue");
                
                // Now add a Client side image map to link directly to task with hovering title
                $bar ->SetCSIMTarget ("../../tasks/task/index.php?proj=$proj&task=".$task_row["id"]."&action=display");
                $bar->SetCSIMAlt(str_replace("\"", "'", $task_row["TaskName"]));
                $bar->title-> SetCSIMTarget("../../tasks/task/index.php?proj=$proj&task=".$task_row["id"]."&action=display");
                $bar->title-> SetCSIMAlt("Click to view details");
                
                $graph->Add($bar);
            }
        }
    }
}

?>
