<?
/*
 * Displays a listing of requests and their associatdtasks for the current user
 * Shows all unresolved requests and those resolved this week, plus tasks resolved in last day
 * Option of viewing all, or tasks due this or last week (plus their requests)
 */ 

include "$CRM_path/tasks/task/task_lib.php";
include "$CRM_path/tasks/calculate_status.inc";

// Converts a week and year to a date
function get_date_from_week($year, $week)
{
    $first_day = strtotime($year."-01-01");
    $is_monday = date("w", $first_day) == 1;
    $is_weekone = strftime("%V", $first_day) == 1;
    if($is_weekone)
    {
        $week_one_start = $is_monday ? $first_day : strtotime("last monday", $first_day);
    }
    else
    {
        $week_one_start = strtotime("next monday", $first_day);
    }
    return $week_one_start+(3600*24*7*($week-1));
}

// Should we be showing a project column as well?
if ($show_project == "on")
{
   CRM_run_query("delete from user_settings where person = $CRM_user and type = 'overview_todo' and name = 'show_project'");
   $show_project = OPT_gpc_slashes($show_project);
   $setting_sql = "insert into user_settings set person = $CRM_user, type = 'overview_todo', name = 'show_project', 
                  value='checked'";
   CRM_run_query($setting_sql);
   $show_project = "checked";
}       
else if (!$scope_filter)  // Means the form wasn't submitted with checkbox off
{  // Get the stored setting version
   $setting_sql="select value from user_settings where person = $CRM_user and type = 'overview_todo' and name = 'show_project'";
   $setting_result = CRM_run_query($setting_sql);
   if (OPT_num_rows($setting_result) == 1)
   {
      $setting_row = OPT_fetch_row($setting_result);
      $show_project = $setting_row[0];
   }
}
else
{  // Form was submitted, but this variable was off, so user wants to change the setting
   CRM_run_query("delete from user_settings where person = $CRM_user and type = 'overview_todo' and name = 'show_project'");
}

// Either save the user's scope filter setting or retrieve it
if ($scope_filter)
{
   $sql="DELETE FROM user_settings where person=$CRM_user and project=0 and type='overview_filters'";
   CRM_run_query($sql);
   $sql="INSERT INTO user_settings (person, project, type, name, value) VALUES   
         ($CRM_user, 0, 'overview_filters', 'scope_filter', '$scope_filter')";
   CRM_run_query($sql);
}
else
{
  $sql="SELECT value FROM user_settings WHERE person=$CRM_user and project=0 and 
        type='overview_filters' and name='scope_filter'";
  $settings_result=CRM_run_query($sql);
  while ($settings_row=OPT_fetch_row($settings_result))
  {
      $scope_filter=intval($settings_row[0]);
  }

  if (!$scope_filter)
  {  // They didn't have filter setting so default to 1
     $scope_filter = 1;  // default is all
  }
}

// Create sql filter matching the user's task filter settings
if ($scope_filter == 1)
{
    $scope_filter_sql=" and ((PercentComplete < 100) OR (FROM_UNIXTIME(EndDate) >= DATE_SUB(NOW(), INTERVAL 1 DAY)))";
}
if ($scope_filter == 2)
{  // Only tasks with deadlines this week
    $week_start=get_date_from_week(date("Y"), date("W"));
    $scope_filter_sql=" and (FROM_UNIXTIME(EndDatePlanned) >= FROM_UNIXTIME($week_start)) and 
                 (FROM_UNIXTIME(EndDatePlanned) < DATE_ADD((FROM_UNIXTIME($week_start)), INTERVAL 7 DAY))";
}
else if ($scope_filter == 3)
{  // Only tasks with deadlines last week
    $week_start=get_date_from_week(date("Y"), date("W"));
    $scope_filter_sql=" and (FROM_UNIXTIME(EndDatePlanned) <= FROM_UNIXTIME($week_start)) and 
                 (FROM_UNIXTIME(EndDatePlanned) > DATE_SUB((FROM_UNIXTIME($week_start)), INTERVAL 7 DAY))";
}

  // And now for the sorting issue	
  if (!isset($order)) {	
    if (!$CRM_guest) {
      $order=read_Xtra("people",$CRM_user,"OPT_todo","order");
    }
    if (!$order) {
      $order="priority";
    }
  } elseif (!$CRM_guest) {
    write_Xtra("people",$CRM_user,"OPT_todo","order",$order);
  }

  // Go get all the requests to display
  $cat_order=OPT_parse_order_string($order);
  $orderby=OPT_build_order_query($cat_order);

  // First we'll use the orderby clause for the requests table with some tweaks for the task table
  $task_orderby = $orderby;
  $task_orderby = str_replace("priority asc", "id asc", $task_orderby);
  $task_orderby = str_replace("priority desc", "id asc", $task_orderby);
  $task_orderby = str_replace("status asc", "id asc", $task_orderby);
  $task_orderby = str_replace("status desc", "id asc", $task_orderby);
  $task_orderby = str_replace("title", "TaskName", $task_orderby);
  $task_orderby = str_replace("deadline", "EndDatePlanned", $task_orderby);
  $task_orderby = str_replace("aproj", "ProjectID", $task_orderby);

if ($scope_filter == 1)
{   
    // First get all requests this user owns
    $sql="select id from requests where (owner = $person) and 
          (status <> 2 || resolved_date > (now() - INTERVAL 1 DAY))";
    $result=CRM_run_query($sql);
    while ($row=OPT_fetch_row($result))
    {
        $req_ids[]=$row[0];  // build up list for IN clause
    }
    
    // Second get all requests not owned by this user, but which have tasks this user is involved with
    $sql="select distinct(requests.id) from requests left join request2task on requests.id = request2task.request, 
      person_task, task where person_task.taskID = request2task.task and person_task.personID = $person
      and request2task.task = task.id and (task.PercentComplete < 100) and requests.owner <> $person";
    $result=CRM_run_query($sql);
    while ($row=OPT_fetch_row($result))
    {
        $req_ids[]=$row[0];  // build up list for IN clause
    }
    
    // Now combine this request list with the correct ordering and filtering
    if ($req_ids)
    {
        $sql="select id, title, priority, status, deadline, aproj, TIME_TO_SEC(initial_estimate)/3600 
              from requests where id IN (". implode(",", $req_ids) . ") $orderby";
        $result=CRM_run_query($sql);
    }  
}
else if (($scope_filter == 2) || ($scope_filter == 3))
{   // Just get requests which have tasks due this (or last) week involving this user
    $sql="select distinct(requests.id), title, priority, status, deadline, aproj, TIME_TO_SEC(initial_estimate)/3600
          from requests, request2task, task, person_task where requests.id = request2task.request and request2task.task = task.id
          and person_task.taskID = task.id and person_task.personID = $person $scope_filter_sql order by priority asc";
    $result=CRM_run_query($sql);
}
startTables();
?>
<form action="." method="post">

<tr>
<td class="regular" bgcolor="<?echo($OPT_col_md);?>" align="center" colspan="2">
<b>Your Global To-Do List (Requests and Tasks)</b></td>
<td class="regular" bgcolor="<?echo($OPT_col_md);?>" align="right" colspan="<?echo($show_project?"6":"5");?>" nowrap>
Show: <select name='scope_filter' onchange="form.submit()">
<option value='1' <? isSel($scope_filter, "1")?>>All Items</option>
<option value='2' <? isSel($scope_filter, "2")?>>Tasks due this week</option>
<option value='3' <? isSel($scope_filter, "3")?>>Tasks due last week</option>
</select>
&nbsp;
<input type='hidden' name='person' value='<?echo($person);?>' />
<input type="checkbox" name="show_project" <?echo($show_project);?> onchange="form.submit()">Show Project
</td>
</tr>
<tr>
<?
echo(OPT_header_cell("id",$cat_order,__("ID"),"person=$person"));
echo(OPT_header_cell("title",$cat_order,__("Title"), "person=$person"));
echo(OPT_header_cell("priority",$cat_order,__("Priority"),"person=$person"));
echo(OPT_header_cell("status",$cat_order,__("Status"),"person=$person"));
echo(OPT_header_cell("TO_DAYS(date)",$cat_order,__("Date"),"person=$person"));
if ($show_project)
{
echo(OPT_header_cell("aproj",$cat_order,__("Project"),"person=$person"));
}
echo(OPT_header_cell("deadline",$cat_order,__("Deadline"),"person=$person"));
?>
<td class="regular" bgcolor="<?echo($OPT_col_dk);?>" align="center" nowrap><b>Hours Left</b></td>
</tr>
<? 
$req_counter=0;
while ($req_row=OPT_fetch_row($result))
{
   // Need to get settings from the project for this request
   $proj=$req_row[5];
   $statii=get_project_setting($proj, "request_status_options");
   $extra_statuses=explode("|", $statii); 
   $priorities = get_project_setting($proj, "request_priority");
   if (!$priorities) {
      $priorities=array();
   }
   $priorities = explode("|",$priorities);
?>

<tr>
<td class="regular" bgcolor="<?echo($OPT_col_md);?>" align="center">
<a class="regular" href="<?echo($level);?>/rt/req_view/?proj=<?echo($proj)?>&req=<?echo($req_row[0])?>"><?echo($req_row[0])?></a></td>
<td class="regular" bgcolor="<?echo($OPT_col_md);?>" align="left">
<a class="regular" href="<?echo($level);?>/rt/req_view/?proj=<?echo($proj)?>&req=<?echo($req_row[0])?>"><?echo(stripslashes($req_row[1]))?></a></td>

<td class="regular" align="center" bgcolor="<?echo($OPT_col_md);?>"><?echo($priorities[$req_row[2]])?></td>

<td class="regular" bgcolor="<?echo($OPT_col_md);?>" align="center">
<?
  switch ($req_row[3])
  {
    case 1: echo("Open"); break;
    case 2: echo("<font color='green'>Resolved</font>"); break;
    default: echo($extra_statuses[$req_row[3]-3]);
  }
?>
</td>
<? 
    // Date
    $date=niceDate("requests","date",$req_row[0], 1);
    echo("  <td class=\"regular\" align=\"center\" bgcolor=$OPT_col_md>$date</td>\n");

    // Project (?)
    if ($show_project)
    {
        echo("<td class='regular' bgcolor='$OPT_col_md' align='center'>");       
        echo("<a href='../projects/Proj_View/?proj=$proj'>");
        echo(getvalue("projects", "name", $proj));
        echo("</a></td>");
    }

    // Deadline
    $deadline=niceDate("requests","deadline",$req_row[0],1);
    if (!$deadline) {
      $deadline="&nbsp;";
    }
    unset($bold);
    if (getvalue("requests","(deadline<now()) and (status!=2)",$req_row[0])) {
      $bold[]="<strong><font color=\"$OPT_col_attention\">";
      $bold[]="</font></strong>";
    } else {
      $bold[]="";
      $bold[]="";
    }
    echo("<td class=\"regular\" align=\"center\" bgcolor=\"$OPT_col_md\">$bold[0]$deadline$bold[1]</td>\n");
?>
<td class="regular" bgcolor="<?echo($OPT_col_md);?>" align="center"> 
<span id="req_hours<?echo($req_counter)?>"></span></td>
</tr>

<?  // Now for each request find all tasks associated with it that match the requested scope filter
$sql="select distinct(task.id), TaskName, StartDatePlanned, EndDatePlanned, EndDate, ProjectID
      from task left join request2task as r2t on r2t.task = task.id, person_task
      where r2t.request=$req_row[0] and person_task.personID = $person and person_task.taskID = task.id
      $scope_filter_sql $task_orderby";
$task_result=CRM_run_query($sql);
// Add in the request estimate if there are no tasks owned by this person and it's not complete
if ((OPT_num_rows($task_result) == 0) && ($req_row[3] != 2))
{
  // Don't use request estimate if there are tasks but none involving this user
  $sql="select count(task.id) from request2task left join task 
        on request2task.task = task.id where request2task.request = $req_row[0]";
  $hours_result = CRM_run_query($sql);
  $hours_row = OPT_fetch_row($hours_result);
  if ($hours_row[0] > 0)
  { // Request has tasks not associated with this user so don't use request estimate
     $req_hours[$req_counter] = 0;  
  }
  else
  { // Use the initial estimate on the request as the hours left
     $req_hours[$req_counter] = $req_row[6];
  }
}
else
{
  $req_hours[$req_counter] = 0;  // add up from tasks!
}

// Now loop on all tasks for this 
while ($task_row=OPT_fetch_row($task_result))
{
?>
<tr>   
<td class="regular" bgcolor="<?echo($OPT_col_lt);?>" align="center">
<a class="regular" href="<?echo($level);?>/tasks/task/?proj=<? echo $proj ?>&task=<? echo($task_row[0])?>&action=display">
<?echo($task_row[0])?></a></td>
<td class="regular" bgcolor="<?echo($OPT_col_lt);?>" align="left">&nbsp;&nbsp;&nbsp;
<a class="regular" href="<?echo($level);?>/tasks/task/?proj=<? echo $proj ?>&task=<? echo($task_row[0])?>&action=display">
<? echo(stripslashes($task_row[1])) ?></a></td>
<td class="regular" bgcolor="<?echo($OPT_col_lt);?>" align="center">&nbsp;</td>
<td class="regular" bgcolor="<?echo($OPT_col_lt);?>" align="center">
<?  // Calculate Task status icon
$StartDatePlanned=$task_row[2];
$EndDatePlanned=$task_row[3];
list($totalcount,$affectcount,$complete,$affect,$personal, $T_time, $T_estimated)=compute_progress($task_row[0]);
$icons = determineStatusIcons($StartDatePlanned, $EndDatePlanned, $complete, $level);
if ($icons)
{
   echo $icons;
}
else
{
   echo("&nbsp;");
}
?>
</td>
<?
    // Date
    $date=niceDate("task","date",$task_row[0], 1);
    echo("  <td class=\"regular\" align=\"center\" bgcolor=$OPT_col_lt>$date</td>\n");

    // Project (?)
    if ($show_project)
    {
        $proj = $task_row[5];
        echo("<td class='regular' bgcolor='$OPT_col_lt' align='center'>");       
        echo("<a href='../projects/Proj_View/?proj=$proj'>");
        echo(getvalue("projects", "name", $proj));
        echo("</a></td>");
    }


    // Task Deadline
    if (!$task_row[3]) {
      $deadline="&nbsp;";
    }
    else
    {
      $deadline=niceDate("task","FROM_UNIXTIME(EndDatePlanned)",$task_row[0],1);
    }
    unset($bold);
    if (getvalue("task","(FROM_UNIXTIME(EndDatePlanned)<now()) and (PercentComplete < 100)",$task_row[0])) {
      $bold[]="<strong><font color=\"$OPT_col_attention\">";
      $bold[]="</font></strong>";
    } else {
      $bold[]="";
      $bold[]="";
    }
    echo("<td class=\"regular\" align=\"center\" bgcolor=\"$OPT_col_lt\">$bold[0]$deadline$bold[1]</td>\n");
?>
<td class="regular" bgcolor="<?echo($OPT_col_lt);?>" align="center">
<?
$task_time_remaining = (100 - $complete)/100 * $T_estimated / 3600;
$req_hours[$req_counter] +=$task_time_remaining;
echo(sprintf("%.0f",$task_time_remaining ));    
?></td>
</tr>
<?
}  // end of while loop for tasks
?>

<?
   $req_counter += 1;
}  // end of while loop for requests


// Now include any tasks not attached to requests that meet the requirements
$sql="select id, TaskName, StartDatePlanned, EndDatePlanned, EndDate, ProjectID
      from person_task, task left join request2task as r2t on r2t.task = task.id
      where r2t.request IS NULL and (person_task.personID = $person and 
      person_task.taskID = task.id) $scope_filter_sql $task_orderby";
$task_result=CRM_run_query($sql);
if (OPT_num_rows($task_result) > 0)
{
  $req_hours[$req_counter] = $req_row[6];
?>
</tr>
<tr>
<td class="regular" bgcolor="<?echo($OPT_col_md);?>" align="center">&nbsp;</td>
<td class="regular" bgcolor="<?echo($OPT_col_md);?>" align="left">Other Tasks (Not attached to a request)</td>
<td class="regular" bgcolor="<?echo($OPT_col_md);?>" align="center">&nbsp;</td>
<td class="regular" bgcolor="<?echo($OPT_col_md);?>" align="center">&nbsp;</td>
<td class="regular" bgcolor="<?echo($OPT_col_md);?>" align="center">&nbsp;</td>
<? if ($show_project) { ?>
<td class="regular" bgcolor="<?echo($OPT_col_md);?>" align="center">&nbsp;</td>
<? } ?>
<td class="regular" bgcolor="<?echo($OPT_col_md);?>" align="center">&nbsp;</td>
<td class="regular" bgcolor="<?echo($OPT_col_md);?>" align="center">
<span id="req_hours<?echo($req_counter)?>"></span></td></td>
</tr>
<?
// Now loop on unattached tasks
while ($task_row=OPT_fetch_row($task_result))
{
?>

<tr>   
<td class="regular" bgcolor="<?echo($OPT_col_lt);?>" align="center">
<a class="regular" href="<?echo($level);?>/tasks/task/?proj=<? echo $proj ?>&task=<? echo($task_row[0])?>&action=display">
<?echo($task_row[0])?></a></td>
<td class="regular" bgcolor="<?echo($OPT_col_lt);?>" align="left">&nbsp;&nbsp;&nbsp;
<a class="regular" href="<?echo($level);?>/tasks/task/?proj=<? echo $proj ?>&task=<? echo($task_row[0])?>&action=display">
<? echo(stripslashes($task_row[1])) ?></a></td>
<td class="regular" bgcolor="<?echo($OPT_col_lt);?>" align="center">&nbsp;</td>
<td class="regular" bgcolor="<?echo($OPT_col_lt);?>" align="center">
<?  // Calculate Task status icon
$StartDatePlanned=$task_row[2];
$EndDatePlanned=$task_row[3];
list($totalcount,$affectcount,$complete,$affect,$personal, $T_time, $T_estimated)=compute_progress($task_row[0]);
$icons = determineStatusIcons($StartDatePlanned, $EndDatePlanned, $complete, $level);
if ($icons)
{
   echo $icons;
}
else
{
   echo("&nbsp;");
}
?>
</td>
<?
    // Date
    $date=niceDate("task","date",$task_row[0], 1);
    echo("  <td class=\"regular\" align=\"center\" bgcolor=$OPT_col_lt>$date</td>\n");
 
    // Project (?)
    if ($show_project)
    {
        $proj = $task_row[5];
        echo("<td class='regular' bgcolor='$OPT_col_lt' align='center'>");       
        echo("<a href='../projects/Proj_View/?proj=$proj'>");
        echo(getvalue("projects", "name", $proj));
        echo("</a></td>");
    }

    // Task Deadline
    if (!$task_row[3]) {
      $deadline="&nbsp;";
    }
    else
    {
      $deadline=niceDate("task","FROM_UNIXTIME(EndDatePlanned)",$task_row[0],1);
    }
    unset($bold);
    if (getvalue("task","(FROM_UNIXTIME(EndDatePlanned)<now()) and (PercentComplete < 100)",$task_row[0])) {
      $bold[]="<strong><font color=\"$OPT_col_attention\">";
      $bold[]="</font></strong>";
    } else {
      $bold[]="";
      $bold[]="";
    }
    echo("  <td class=\"regular\" align=\"center\" bgcolor=\"$OPT_col_lt\">$bold[0]$deadline$bold[1]</td>\n");
?>
<td class="regular" bgcolor="<?echo($OPT_col_lt);?>" align="center">
<?
$task_time_remaining = (100 - $complete)/100 * $T_estimated / 3600;
$req_hours[$req_counter] +=$task_time_remaining;
echo(sprintf("%.0f",$task_time_remaining ));  
?>

</td>
</tr>

<?

}  // end of while loop on unattached tasks
$req_counter += 1;
}  // end of if some rows


?>

<script language="JavaScript" type="text/javascript">
<!--
<?  // Now fill in the sub-totals for each category/priority heading
for ($i=0; $i<$req_counter; $i++)
{
    echo("req_hour=window.document.getElementById('req_hours".$i."');\n");
    echo("req_hour.innerHTML=\"".sprintf("%.0f", $req_hours[$i])."\";\n");
    $total_hours += $req_hours[$i];
}

?>
//-->
</script>

<tr>
<td class="regular" bgcolor="<?echo($OPT_col_dk);?>" align="center">&nbsp;</td>
<td class="regular" bgcolor="<?echo($OPT_col_dk);?>" align="left"><b>Effort Remaining</b> (Total Hours)</td>
<td class="regular" bgcolor="<?echo($OPT_col_dk);?>" align="center">&nbsp;</td>
<td class="regular" bgcolor="<?echo($OPT_col_dk);?>" align="center">&nbsp;</td>
<td class="regular" bgcolor="<?echo($OPT_col_dk);?>" align="center">&nbsp;</td>
<? if ($show_project) { ?>
<td class="regular" bgcolor="<?echo($OPT_col_dk);?>" align="center">&nbsp;</td>
<? } ?>
<td class="regular" bgcolor="<?echo($OPT_col_dk);?>" align="center">&nbsp;</td>
<td class="regular" bgcolor="<?echo($OPT_col_dk);?>" align="center">
<b><?echo(sprintf("%.0f", $total_hours))?></b></td>
</tr>
</form>
<?
endTables();
?>
