<?
include ("$CRM_path/tasks/task/task_lib.php");  // need for computing task progress

/* Determines the following values for a specific request based on any 
 * tasks associated with it that have valid start or end dates
 * Only on tasks that are not sub-tasks as they're already included
 * Return array of [start, end, progress, caption]
 */
function calcRequestDetails($req, $req_date, $req_deadline)
{
    $start = INT_MAX;
    $end = 0;
    
    $sql="select task.id as id, StartDate, StartDatePlanned, 
      EndDate, EndDatePlanned, PercentComplete from request2task, task
      where request2task.task = task.id and request = $req
      and (StartDate > 0 OR StartDatePlanned > 0) and 
          (EndDate > 0 OR EndDatePlanned > 0) and parent = 0"; 
    $result = CRM_run_query($sql);
    while ($row = OPT_fetch_array($result))
    {
        // See if we need to descrease the start date
        if (($row["StartDate"] < $start) && ($row["StartDate"] > 0))
        {
            $start = $row["StartDate"];
        }
        else if (($row["StartDatePlanned"] < $start) && ($row["StartDatePlanned"] > 0))
        {
            $start = $row["StartDatePlanned"];
        }       
        
        // See if we need to increase the end date
        if ($row["EndDate"] > $end)
        {
            $end = $row["EndDate"];
        }
        else if ($row["EndDatePlanned"] > $end)
        {
            $end = $row["EndDatePlanned"];
        } 
        
        // Calculate the progress of this top-level task
        list($totalcount,$affectcount,$complete,$affect,$personal,$time,$estimated)=compute_progress($row["id"]);
        
        // This is the number of hours already completed
        $complete_hours += ($complete * $estimated / 100);
        
        // This is the total time to complete
        $total_hours += $estimated;
    }
    
    // Now convert to proper format for JpGraph
    if ($start == INT_MAX)
    {
        $start = $req_date;  // Use the creation time of the request
    }
    $start_date=date("Y-m-d", $start);

    
    if (!$end)
    {   // If end date is still empty, use the deadline possibly set for it
        $end_date=date("Y-m-d", $req_deadline);
    }
    else
    {
        $end_date=date("Y-m-d", $end);
    }

    // Determine progress so far based on weighting of time
    if ($total_hours > 0)
    {
        $progress = $complete_hours / $total_hours;
        if ($progress < 1)
        {
            $caption = "[" . sprintf("%.0f", ($progress * 100)) . "%]";
        }
    }
    else
    {   
        $progress=0;
    }

    return array($start_date, $end_date, $progress, $caption);
}

?>