<?
// Generates a filter string of SQL based on all the fields selected
// by the user during an advanced search


// ******************* Attributes section **************************
if ($selected_priorities)
{
    if ($priority_modifier == 0) { $modifier = "NOT"; } else { unset($modifier); }
    $filter_components[] = "priority $modifier IN (". implode(",", $selected_priorities) . ")";
}

if ($selected_statuses)
{
    if ($status_modifier == 0) { $modifier = "NOT"; } else { unset($modifier); }
    $filter_components[] = "status $modifier IN (". implode(",", $selected_statuses) . ")";
}

if ($selected_categories)
{
    if ($category_modifier == 0) { $modifier = "NOT"; } else { unset($modifier); }
    $filter_components[] = "CatID $modifier IN (". implode(",", $selected_categories) . ")";
}

if ($selected_milestones)
{
    if ($milestone_modifier == 0) { $modifier = "NOT"; } else { unset($modifier); }
    $filter_components[] = "milestone $modifier IN (". implode(",", $selected_milestones) . ")";
}


// ******************* String section **************************
if ($title_string && $title_string_type)
{
    $filter_components[] = "(" .get_string_search_filter($title_string_type, $title_string, "title") .")";
}

if ($description_string && $description_string_type)
{
    $filter_components[] = "(" .get_string_search_filter($description_string_type, $description_string, "request") .")";
}

if ($comment_string && $comment_string_type)
{
    $comment_table_filter = get_string_search_filter($comment_string_type, $comment_string, "comment");
    $sql = "select request from request_answers where $comment_table_filter";
    $result = crm_run_query($sql);
    while ($row = OPT_fetch_row($result))
    {
        $comment_request_ids[] = $row[0];
    }
    if ($comment_request_ids)
    {
        $filter_components[] = "id IN (". implode(",", $comment_request_ids) .")";
    }
    else
    {
        $filter_components[] = "1=0";  // No comments met this criteria so no requests should be found
    }
}

if ($forum_string && $forum_string_type)
{
    $forum_table_filter_for_title = get_string_search_filter($forum_string_type, $forum_string, "title");
    $forum_table_filter_for_content = get_string_search_filter($forum_string_type, $forum_string, "content");
    $sql = "select att_id from forums where att_type = 'requests' and $forum_table_filter_for_title OR $forum_table_filter_for_content";
    $result = crm_run_query($sql);
    while ($row = OPT_fetch_row($result))
    {
        $forum_request_ids[] = $row[0];
    }
    if ($forum_request_ids)
    {
        $filter_components[] = "id IN (". implode(",", $forum_request_ids) .")";
    }
    else
    {
        $filter_components[] = "1=0";  // No forum entries met this criteria so no requests should be found
    }
}


// ******************* People section **************************
if (sizeOf($request_owner) && ($request_owner > -1))
{  
   unset($modifier);
   if (!$owner_modifier)
   {
      $modifier="NOT";
   }  
   $filter_components[] = "$modifier (owner = ".abs($request_owner) .")";     
}

if ($request_submitter > 0)
{
   unset($modifier);
   if (!$submitter_modifier)
   {
      $modifier="NOT";
   }
   $sql="select request from request_submitters where (person = ".abs($request_submitter).")";
   $result=CRM_run_query($sql);
   while ($row=OPT_fetch_row($result))
   {
      $submitter_request_ids[] = $row[0];
   }
   if ($submitter_request_ids)
   {
        $filter_components[] = "id $modifier IN (". implode(",", $submitter_request_ids) .")";
   }
   else
   {
        $filter_components[] = "$modifier 1=0";  // No requests submitted (or not) by this individual
   }
}

if ($request_commenter > 0)
{
   unset($modifier);
   if (!$commenter_modifier)
   {
      $modifier="NOT";
   }
   $sql="select request from request_answers where $modifier (src = ".abs($request_commenter).")";
   $result=CRM_run_query($sql);
   while ($row=OPT_fetch_row($result))
   {
      $commenter_request_ids[] = $row[0];
   }
   if ($commenter_request_ids)
   {
        $filter_components[] = "id IN (". implode(",", $commenter_request_ids) .")";
   }
   else
   {
        $filter_components[] = "$modifier 1=0";  // No requests submitted (or not) by this individual
   }
}

if ($request_forum_submitter > 0)
{
   unset($modifier);
   if (!$forum_submitter_modifier)
   {
      $modifier="NOT";
   }
   $sql="select att_id from forums where att_type= 'requests' and (src = ".abs($request_forum_submitter).")";
   $result=CRM_run_query($sql);
   while ($row=OPT_fetch_row($result))
   {
      $forum_submitter_request_ids[] = $row[0];
   }
   if ($forum_submitter_request_ids)
   {
        $filter_components[] = "id $modifier IN (". implode(",", $forum_submitter_request_ids) .")";
   }
   else
   {
        $filter_components[] = "$modifier 1=0";  // No requests submitted (or not) by this individual
   }
}


// ******************* Dates section **************************
if (strlen($deadline_date_start) && (strlen($deadline_date_end) &&
   (strtotime($deadline_date_start) > 0) && (strtotime($deadline_date_end) > 0)))
{
   $filter_components[] = "(deadline > '$deadline_date_start' AND deadline < DATE_ADD('$deadline_date_end', INTERVAL 1 DAY))";
}
if (strlen($creation_date_start) && (strlen($creation_date_end) &&
   (strtotime($creation_date_start) > 0) && (strtotime($creation_date_end) > 0)))
{
   $filter_components[] = "(date > '$creation_date_start' AND date < DATE_ADD('$creation_date_end', INTERVAL 1 DAY))";
}
if (strlen($resolution_date_start) && (strlen($resolution_date_end) &&
   (strtotime($resolution_date_start) > 0) && (strtotime($resolution_date_end) > 0)))
{
   $filter_components[] = "(resolved_date > '$resolution_date_start' AND resolved_date < DATE_ADD('$resolution_date_end', INTERVAL 1 DAY))";
}
if (strlen($hours_estimated_low) && strlen($hours_estimated_high))
{
   $filter_components[] = "(TIME_TO_SEC(initial_estimate) >= '".OPT_gpc_slashes(3600*$hours_estimated_low)."' AND TIME_TO_SEC(initial_estimate) <= '".OPT_gpc_slashes(3600*$hours_estimated_high)."')";  
}






// Fake out if no criteria selected at all; also means that completely empty form fields mean include all!
if ($filter_components)
{
   // Need a leading AND as first clause limits based on 'aproj = $proj' automatically...
   $filter = "AND " . implode(" AND ", $filter_components);
}








// Generates the filter from a string search on title or description
function get_string_search_filter($type, $text, $field)
{
    // Handle the particular search type requested
    if (0 == strlen($text))
    { 
        $filter = "1 = 0";  // No text provided
    }
    else if ($type == "substring")
    {
        $substring=OPT_gpc_slashes($text);
        $filter = "$field like '%$substring%'";
    }
    else if ($type == "allwords")
    {
        $words=explode(" ", $text);
        if (sizeOf($words) > 0)
        {
            for ($i=0; $i<sizeOf($words); $i++)
            {
                $word = OPT_gpc_slashes($words[$i]);
                $clauses[] =  "$field like '%$word%'";
            }
            $filter = $filter . implode(" and ", $clauses);
        }
        else
        {  // just spaces provided; they get no results
            $filter = $filter ." 1 = 0";
        }
    }
    else if ($type == "anywords")
    {
        $words=explode(" ", $text);
        if (sizeOf($words) > 0)
        {
            for ($i=0; $i<sizeOf($words); $i++)
            {
                $word = OPT_gpc_slashes($words[$i]);
                $clauses[] =  "$field like '%$word%'";
            }
            $filter = implode(" or ", $clauses);
        }
        else
        {   // just spaces provided; they get no results
            $filter = " 1 = 0";
        }
    }
    else if ($type == "regexp")
    {
        $expr=OPT_gpc_slashes($text);
        $filter = "$field regexp '$expr'";
    }
    else if ($type == "notregexp")
    {
        $expr=OPT_gpc_slashes($text);
        $filter = "$field not regexp '$expr'";
    }
    return $filter;
}


?>