<?php
// Displays a line plot of estimated hours of effort remaining against time 
// spent working on the task  (line should drop :)

include ("../../include/OPT_lib.php");
auth_open();
include ("$CRM_inc/jpgraph/jpgraph.php");
include ("$CRM_inc/jpgraph/jpgraph_line.php");

// First we determine all the timestamps when a progress change occured for a task associated with this request
$prog_sql="select unix_timestamp(task_progress_history.date) from task_progress_history, request2task
           where taskID=request2task.task and request2task.request=$req order by date";
$prog_result=crm_run_query($prog_sql);
while($prog_row=OPT_fetch_row($prog_result))
{
    $xdata[] = $prog_row[0];
}

// Get the list of tasks associated with this request
$sql="select distinct(task) from request2task where request = $req";
$task_result=crm_run_query($sql);
while ($task_row=OPT_fetch_row($task_result))
{
    $task_ids[] = $task_row[0];
}

$ymax=0;

// Now for each time stamp, we need to determine the hours remaining or percentage complete
// of all tasks associated with this request at that time point
if (get_project_setting($proj, "task_status_hours_remaining") == "1")
{
    $yaxis_title="Effort Remaining (Hours)";
    // For each time point
    for ($i=0; $i<sizeOf($xdata); $i++)
    {   // Get the remaining hours for this task    
        $hrs_remaining=0;
        for ($j=0; $j<sizeOf($task_ids); $j++)
        {
            $sql="select remaining from task_progress_history where unix_timestamp(date) <= $xdata[$i] 
                  and taskID=$task_ids[$j]
                  order by date desc limit 1";
            $rem_result=crm_run_query($sql);
            if (OPT_num_rows($rem_result) == 1)
            {
                $rem_row=OPT_fetch_row($rem_result);
                $hrs_remaining=$hrs_remaining+$rem_row[0];
            }
        }
        if ($ymax < $hrs_remaining)
        {
            $ymax = $hrs_remaining;
        }
        $ydata[] = $hrs_remaining;
    }
}
else  // we're interested in percentage complete
{
    // FIXME!  How should this be displayed?  Weighted?
    $yaxis_title="Percentage Complete";
}

$startTime=$xdata[0];
$endTime=$xdata[sizeOf($xdata)-1];

// The callback that converts timestamp to human readable format
function DateTimeCallback($aVal) {
    return Date("M d\nH:i",$aVal);
} 

// Create the graph. These two calls are always required
$graph = new Graph(700,300,"auto");   
$graph->SetScale("linlin",0,$ymax+($ymax/10),$startTime,$endTime);
$graph->SetShadow();

// Setup the x-axis with a format callback to convert the timestamp
// to a user readable time
$graph->xaxis->SetLabelFormatCallback('DateTimeCallback');
$graph->xaxis->scale->ticks->Set(($endTime-$startTime)/7); 
//$graph->xaxis->HideFirstTicklabel();

// Setup margin and titles
$graph->img->SetMargin(60,20,10,40);

// Setup font for axis
$graph->xaxis->SetFont(FF_VERDANA,FS_NORMAL,8);

// Setup the Y-Axis
$graph->yaxis->title->Set($yaxis_title);

// Show more resolution below 10 hours
if ($ymax <= 10)
{
    $graph->yaxis->SetLabelFormat('%0.1f');
}

// Set Y-axis to only have roughly 10 ticks
if ($ymax <= 1)
{
    $graph->yaxis->scale->ticks->Set(0.1,0.05);
}
else if ($ymax <= 5)
{
    $graph->yaxis->scale->ticks->Set(0.5,0.25);
}
else if ($ymax <= 10)
{
    $graph->yaxis->scale->ticks->Set(1,0.5);
}
else if ($ymax <= 50)
{
    $graph->yaxis->scale->ticks->Set(5,2.5);
}
else if ($ymax <= 100) 
{
    $graph->yaxis->scale->ticks->Set(10,5);
}
else if ($ymax <= 200)
{
    $graph->yaxis->scale->ticks->Set(20,10);
}
else if ($ymax <= 300)
{
    $graph->yaxis->scale->ticks->Set(30,15);
}
else if ($ymax <= 400)
{
    $graph->yaxis->scale->ticks->Set(40,20);
}
else if ($ymax <= 500)
{
    $graph->yaxis->scale->ticks->Set(50,25);
}
else if ($ymax <= 600)
{
    $graph->yaxis->scale->ticks->Set(60,30);
}
else if ($ymax <= 700)
{
    $graph->yaxis->scale->ticks->Set(70,35);
}
else if ($ymax <= 800)
{
    $graph->yaxis->scale->ticks->Set(80,40);
}
else if ($ymax <= 900)
{
    $graph->yaxis->scale->ticks->Set(90,45);
}
else
{
    $graph->yaxis->scale->ticks->Set(100,50);
}

// Create the linear plot
$lineplot=new LinePlot($ydata,$xdata);
$lineplot->SetFillColor("royalblue3");

// Add the plot to the graph
$graph->Add($lineplot);

// Display the graph
$graph->Stroke();

auth_close();
?>