<?
// Generates a filter string of SQL based on all the fields selected
// by the user during an advanced search


// ******************* Attributes section **************************
if ($selected_statuses)
{
    if ($status_modifier == 0) { $modifier = "NOT"; } else { unset($modifier); }
    $status_query = get_status_query($selected_statuses);
    $filter_components[] = "$modifier $status_query";
}

if ($selected_categories)
{
    if ($category_modifier == 0) { $modifier = "NOT"; } else { unset($modifier); }
    $filter_components[] = "CatID $modifier IN (". implode(",", $selected_categories) . ")";
}


// ******************* String section **************************
if ($title_string && $title_string_type)
{
    $filter_components[] = "(" .get_string_search_filter($title_string_type, $title_string, "TaskName") .")";
}

if ($description_string && $description_string_type)
{
    $filter_components[] = "(" .get_string_search_filter($description_string_type, $description_string, "TaskDesc") .")";
}

if ($comment_string && $comment_string_type)
{
    $comment_table_filter = get_string_search_filter($comment_string_type, $comment_string, "record");
    $sql = "select taskID from task_history where $comment_table_filter";
    $result = crm_run_query($sql);
    while ($row = OPT_fetch_row($result))
    {
        $comment_task_ids[] = $row[0];
    }
    if ($comment_task_ids)
    {
        $filter_components[] = "id IN (". implode(",", $comment_task_ids) .")";
    }
    else
    {
        $filter_components[] = "1=0";  // No comments met this criteria so no tasks should be found
    }
}

if ($forum_string && $forum_string_type)
{
    $forum_table_filter_for_title = get_string_search_filter($forum_string_type, $forum_string, "title");
    $forum_table_filter_for_content = get_string_search_filter($forum_string_type, $forum_string, "content");
    $sql = "select att_id from forums where att_type = 'task' and $forum_table_filter_for_title OR $forum_table_filter_for_content";
    $result = crm_run_query($sql);
    while ($row = OPT_fetch_row($result))
    {
        $forum_task_ids[] = $row[0];
    }
    if ($forum_task_ids)
    {
        $filter_components[] = "id IN (". implode(",", $forum_task_ids) .")";
    }
    else
    {
        $filter_components[] = "1=0";  // No forum entries met this criteria so no tasks should be found
    }
}


// ******************* People section **************************
if (sizeOf($task_owner) && ($task_owner > -1))
{  
   unset($modifier);
   if (!$owner_modifier)
   {
      $modifier="NOT";
   }  

   $sql="select taskID from person_task where (owner = 'Y') and (personID = ".abs($task_owner).")";
   $result=CRM_run_query($sql);
   while ($row=OPT_fetch_row($result))
   {
      $owner_task_ids[] = $row[0];
   }
   if ($owner_task_ids)
   {
        $filter_components[] = "id $modifier IN (". implode(",", $owner_task_ids) .")";
   }
   else
   {
        $filter_components[] = "1=0";  // No tasks submitted (or not) by this individual
   }
}

if ($task_resource > 0)
{
   unset($modifier);
   if (!$resource_modifier)
   {
      $modifier="NOT";
   }
   $sql="select taskID from person_task where (personID = ".abs($task_resource).")";
   $result=CRM_run_query($sql);
   while ($row=OPT_fetch_row($result))
   {
      $resource_task_ids[] = $row[0];
   }
   if ($resource_task_ids)
   {
        $filter_components[] = "id $modifier IN (". implode(",", $resource_task_ids) .")";
   }
   else
   {
        $filter_components[] = "$modifier 1=0";  // No tasks submitted (or not) by this individual
   }
}

if ($task_commenter > 0)
{
   unset($modifier);
   if (!$commenter_modifier)
   {
      $modifier="NOT";
   }
   $sql="select taskID from task_history where (authorID = ".abs($task_commenter).")";
   $result=CRM_run_query($sql);
   while ($row=OPT_fetch_row($result))
   {
      $commenter_task_ids[] = $row[0];
   }
   if ($commenter_task_ids)
   {
        $filter_components[] = "id $modifier IN (". implode(",", $commenter_task_ids) .")";
   }
   else
   {
        $filter_components[] = "$modifier 1=0";  // No tasks submitted (or not) by this individual
   }
}

if ($task_forum_submitter > 0)
{
   unset($modifier);
   if (!$forum_submitter_modifier)
   {
      $modifier="NOT";
   }
   $sql="select att_id from forums where att_type= 'tasks' and (src = ".abs($task_forum_submitter).")";
   $result=CRM_run_query($sql);
   while ($row=OPT_fetch_row($result))
   {
      $forum_submitter_task_ids[] = $row[0];
   }
   if ($forum_submitter_task_ids)
   {
        $filter_components[] = "id $modifier IN (". implode(",", $forum_submitter_task_ids) .")";
   }
   else
   {
        $filter_components[] = "$modifier 1=0";  // No tasks submitted (or not) by this individual
   }
}


// ******************* Dates section **************************
if (strlen($planned_start_date_start) && (strlen($planned_start_date_end) &&
   (strtotime($planned_start_date_start) > 0) && (strtotime($planned_start_date_end) > 0)))
{
   $filter_components[] = "(StartDatePlanned > '$planned_start_date_start' AND StartDatePlanned < DATE_ADD('$planned_start_date_end', INTERVAL 1 DAY))";
}
if (strlen($planned_end_date_start) && (strlen($planned_end_date_end) &&
   (strtotime($planned_end_date_start) > 0) && (strtotime($planned_end_date_end) > 0)))
{
   $filter_components[] = "(EndDatePlanned > '$planned_end_date_start' AND EndDatePlanned < DATE_ADD('$planned_end_date_end', INTERVAL 1 DAY))";
}
if (strlen($actual_start_date_start) && (strlen($actual_start_date_end) &&
   (strtotime($actual_start_date_start) > 0) && (strtotime($actual_start_date_end) > 0)))
{
   $filter_components[] = "(StartDate > '$actual_start_date_start' AND StartDate < DATE_ADD('$actual_start_date_end', INTERVAL 1 DAY))";
}
if (strlen($actual_end_date_start) && (strlen($actual_end_date_end) &&
   (strtotime($actual_end_date_start) > 0) && (strtotime($actual_end_date_end) > 0)))
{
   $filter_components[] = "(EndDate > '$actual_end_date_start' AND EndDate < DATE_ADD('$actual_end_date_end', INTERVAL 1 DAY))";
}
if (strlen($creation_date_start) && (strlen($creation_date_end) &&
   (strtotime($creation_date_start) > 0) && (strtotime($creation_date_end) > 0)))
{
   $filter_components[] = "(date > '$creation_date_start' AND date < DATE_ADD('$creation_date_end', INTERVAL 1 DAY))";
}
if (strlen($update_date_start) && (strlen($update_date_end) &&
   (strtotime($update_date_start) > 0) && (strtotime($update_date_end) > 0)))
{
   $filter_components[] = "(StatusUpdated > '$update_date_start' AND StatusUpdated < DATE_ADD('$update_date_end', INTERVAL 1 DAY))";
}



// ******************* Effort/progress section **************************
if (strlen($estimate_low) && (strlen($estimate_high)))
{
    $estimate_low = 3600 * floatval($estimate_low);
    $estimate_high = 3600 * floatval($estimate_high);
    $filter_components[] = "((TIME_TO_SEC(estimated) >= $estimate_low) AND ((TIME_TO_SEC(estimated) <= $estimate_high)))";
}
if (strlen($worked_low) && (strlen($worked_high)))
{
    $worked_low = 3600 * floatval($worked_low);
    $worked_high = 3600 * floatval($worked_high);
    $sql="select taskID, sum(TIME_TO_SEC(worked)) as worked_secs from task_progress_history group by taskID";
    $result = crm_run_query($sql);
    while ($row = OPT_fetch_row($result))
    { 
        if (($worked_low <= $row[1]) && ($worked_high >= $row[1]))
        {
            $worked_task_ids[] = $row[0];
        }
    }
    if ($worked_task_ids)
    {
        $filter_components[] = "id $modifier IN (". implode(",", $worked_task_ids) .")";
    }
    else
    {
        $filter_components[] = "1=0";  // No tasks submitted (or not) by this individual
    }
}
if (strlen($remaining_low) && (strlen($remaining_high)))
{
    $remaining_low = 3600 * floatval($remaining_low);
    $remaining_high = 3600 * floatval($remaining_high);
    $filter_components[] = "(((TIME_TO_SEC(estimated)*(100-PercentComplete)/100) >= $remaining_low) AND ((TIME_TO_SEC(estimated)*(100-PercentComplete)/100) <= $remaining_high))";
}
if (strlen($complete_low) && (strlen($complete_high)))
{
    $complete_low = abs($complete_low);
    $complete_high = abs($complete_high);
    $filter_components[] = "((PercentComplete >= $complete_low) AND (PercentComplete <= $complete_high))";
}







// Fake out if no criteria selected at all; also means that completely empty form fields mean include all!
if ($filter_components)
{
   // Need a leading AND as first clause limits based on 'aproj = $proj' automatically...
   $filter = "AND " . implode(" AND ", $filter_components);
}








// Generates the filter from a string search on title or description
function get_string_search_filter($type, $text, $field)
{
    // Handle the particular search type requested
    if (0 == strlen($text))
    { 
        $filter = "1 = 0";  // No text provided
    }
    else if ($type == "substring")
    {
        $substring=OPT_gpc_slashes($text);
        $filter = "$field like '%$substring%'";
    }
    else if ($type == "allwords")
    {
        $words=explode(" ", $text);
        if (sizeOf($words) > 0)
        {
            for ($i=0; $i<sizeOf($words); $i++)
            {
                $word = OPT_gpc_slashes($words[$i]);
                $clauses[] =  "$field like '%$word%'";
            }
            $filter = $filter . implode(" and ", $clauses);
        }
        else
        {  // just spaces provided; they get no results
            $filter = $filter ." 1 = 0";
        }
    }
    else if ($type == "anywords")
    {
        $words=explode(" ", $text);
        if (sizeOf($words) > 0)
        {
            for ($i=0; $i<sizeOf($words); $i++)
            {
                $word = OPT_gpc_slashes($words[$i]);
                $clauses[] =  "$field like '%$word%'";
            }
            $filter = implode(" or ", $clauses);
        }
        else
        {   // just spaces provided; they get no results
            $filter = " 1 = 0";
        }
    }
    else if ($type == "regexp")
    {
        $expr=OPT_gpc_slashes($text);
        $filter = "$field regexp '$expr'";
    }
    else if ($type == "notregexp")
    {
        $expr=OPT_gpc_slashes($text);
        $filter = "$field not regexp '$expr'";
    }
    return $filter;
}

// Determines the SQL query to use based on which status possibilities were selected
function get_status_query($selected_statuses)
{
    for ($i=0; $i<sizeOf($selected_statuses); $i++)
    {
        if (abs($selected_statuses[$i]) == 0)
        { // Task is "In progress"
            $sql_conds[] = "((PercentComplete > 0) and (PercentComplete < 100) and (EndDatePlanned < DATE_ADD(now(), INTERVAL 1 DAY)))";
        }
        else if (abs($selected_statuses[$i]) == 1)
        { // Task is "In progress, overdue"
            $sql_conds[] = "((PercentComplete > 0) and (PercentComplete < 100) and (EndDatePlanned >= DATE_ADD(now(), INTERVAL 1 DAY)))";
        }
        else if (abs($selected_statuses[$i]) == 2)
        { // Task is "Awaiting"
            $sql_conds[] = "((PercentComplete = 0) and (StartDatePlanned > 0) and (StartDatePlanned < DATE_ADD(now(), INTERVAL 1 DAY)))";
        }
        else if (abs($selected_statuses[$i]) == 3)
        { // Task is "Start date missed"
            $sql_conds[] = "((PercentComplete = 0) and (StartDatePlanned > 0) and (StartDatePlanned >= DATE_ADD(now(), INTERVAL 1 DAY)))";
        }
        else if (abs($selected_statuses[$i]) == 4)
        { // Task is "Start and end date both missed"
            $sql_conds[] = "((PercentComplete = 0) and (StartDatePlanned > 0) and (StartDatePlanned >= DATE_ADD(now(), INTERVAL 1 DAY)) and (EndDatePlanned > 0) and (EndDatePlanned >= DATE_ADD(now(), INTERVAL 1 DAY)))";
        }
        else if (abs($selected_statuses[$i]) == 5)
        { // Task is "Complete"
            $sql_conds[] = "(PercentComplete = 100)";
        }
    }
    
    // Now generate a big OR list of conditions
    if ($sql_conds)
    {
        return "(" . implode(" OR ", $sql_conds) . ")";
    }
    else
    {
        return "1=1";
    }
}


?>